ifndef bipedPhy_wdl;
define bipedPhy_wdl;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.00
// date: 060510
//
// title: Biped Physics (01)
// class: MOVEMENT
// type: SUPPORT
// image: bipedPhy.pcx
// help: Handle the physics of motion for bipeds in the world.
//
// Fixed duck bug where entity with center pulls legs up to duck
// added run state not just walk moving fast.
// added fall damage
// improved code throught
//
// ENDHEADER
// ---------------------------------------------------------------------



// =====================================================================
// USER DEFINED VALUES

// section: Friction

// entry: "Glide"
// help: Friction for gliding along surfaces (0..1)
// id: 10
var	move_friction = 0.25;              // engine var used by c_move

// entry: Ground
// help: Friction while on solid ground
// id: 11
var	bipedPhy_grnd_friction = 0.5;

// entry: Ground Angular
// help: Effects panning on ground
// id: 12
var	bipedPhy_grnd_ang_fric = 1.0;        //---

// entry: Air
// help: Friction used when jumping/falling.
// id: 13
var	bipedPhy_air_friction = 0.1;

// entry: Water
// help: Friction used while swimming
// id: 14
var	bipedPhy_water_friction = 1.65;
// entry: Water Angular
// help: Effects panning in water
// id: 15
var	bipedPhy_water_ang_friction = 1.65;


//---------------------------------------------------
// section: Environmental Forces

// enable: Use Passable as Water?
// help: If set, bipeds treat passable blocks as 'water'
// id: 23
define	bipedPhy_passable_water;


// entry: Float force
// help: this is the amount of upward force applied to bipeds in a passible surface.
// id: 16
var	bipedPhy_water_float = 1.85;


// entry: Gravity
// help: Downward pull on bipeds (keeps them from floating away)
// id: 17
var	bipedPhy_gravity = 9;




//----------------------------------------------------------------------
// section: Damage from Falling

// entry: Safe Time
// help: Maximum time biped can fall without damage (ticks)
// id: 19
define bipedPhy_safe_fall_time_const = 9;

// entry: Damage Const
// help: Constant used to calculate damage from falling
// id: 20
define bipedPhy_fall_damage_const = 1.75;

//----------------------------------------------------------------------
// section: Damage from drowning

// entry: Safe Time
// help: Time underwater before taking damage (ticks)
// id: 21
define	bipedPhy_safe_underwater_time_const = 220;

// entry: Damage
// help: Damage per tick while drowning
// id: 22
define	bipedPhy_drowning_damage_const = 0.125;


//----------------------------------------------------------------------
// "Local" global values (used for parameter passing)
var	bipedPhy_height;						// height from the bottom of the model to the nearest solid surface.
//var	bipedPhy_passable_height;			// height from the bottom of the model to the nearest passable surface. (0 if none)
var	bipedPhy_surface_normal_vec[3];	// surface normal of the surface beneight the ent
var	bipedPhy_surface_vel_vec[3];		// the velocity of the surface under the ent
var	bipedPhy_force_vec[3];
var	bipedPhy_ang_vec[3];

define	bipedPhy_scan_dist_const = 96;	// max distance we scan under our biped.

entity* bipedPhy_temp_ent;					// entity under biped
entity* bipedPhy_content_ent;				// entity biped is in




ifdef	develop;
var bipedPhy_debug_data[22];

text bipedPhy_debug_txt
{
     font = standard_font;
     pos_x = 430;
     pos_y = 1;
     string = 
     		"Scan Results (C_Collide)",
     		"========================"
     		" <--height(*1000)",
     		" <--trace_hit",
     		" <--state",
     		" <--IN_SOLID ON_PASSABLE IN_PASSABLE",
     		" <--bipedPhy_surface_normal_vec.x",
     		" <--bipedPhy_surface_normal_vec.y",
     		" <--bipedPhy_surface_normal_vec.z",
     		" <--bipedPhy_force_vec.x",
     		" <--bipedPhy_force_vec.y",
     		" <--bipedPhy_force_vec.z",
     		" <--bipedPhy_ang_vec.x",
     		" <--bipedPhy_ang_vec.y",
     		" <--bipedPhy_ang_vec.z",
     		" <--dist.x",
     		" <--dist.y",
     		" <--dist.z",
     		" <--abs_dist.x",
     		" <--abs_dist.y",
     		" <--abs_dist.z",
     		" <--my.GL_biped_dist"
     		" <--ang_dist",
     		" <--ang_temp";
     flags = d3d;
}


panel bipedPhy_debug_pan
{
     pos_x = 400;
     pos_y = 10;
     digits = 0, 9,  5, standard_font, 1000, bipedPhy_debug_data[0];  //   height
     digits = 0, 18, 5, standard_font, 1, bipedPhy_debug_data[1];			// trace_hit
     digits = 0, 27, 5, standard_font, 1, bipedPhy_debug_data[2]; 		// state
     digits = 0, 36, 5, standard_font, 1, bipedPhy_debug_data[3]; //
     digits = 0, 45, 5, standard_font, 1, bipedPhy_debug_data[4]; //
     digits = 0, 54, 5, standard_font, 100, bipedPhy_debug_data[5]; // normal.x
     digits = 0, 63, 5, standard_font, 100, bipedPhy_debug_data[6]; // normal.y
     digits = 0, 72, 5, standard_font, 100, bipedPhy_debug_data[7]; // normal.z
     digits = 0, 81, 5, standard_font, 100, bipedPhy_debug_data[8]; // you
     digits = 0, 90, 5, standard_font, 100, bipedPhy_debug_data[9]; //
     digits = 0, 99, 5, standard_font, 100, bipedPhy_debug_data[10]; //dist
     digits = 0, 108, 5, standard_font, 100, bipedPhy_debug_data[11];   // dist / time
     digits = 0, 117, 5, standard_font, 100, bipedPhy_debug_data[12];
     digits = 0, 126, 5, standard_font, 100, bipedPhy_debug_data[13];
     digits = 0, 135, 5, standard_font, 100, bipedPhy_debug_data[14];
     digits = 0, 144, 5, standard_font, 100, bipedPhy_debug_data[15];
     digits = 0, 153, 5, standard_font, 100, bipedPhy_debug_data[16];
     digits = 0, 162, 5, standard_font, 100, bipedPhy_debug_data[17];
     digits = 0, 171, 5, standard_font, 100, bipedPhy_debug_data[18];
     digits = 0, 180, 5, standard_font, 100, bipedPhy_debug_data[19];
     digits = 0, 189, 5, standard_font, 100, bipedPhy_debug_data[20];
     digits = 0, 198, 5, standard_font, 100, bipedPhy_debug_data[21];
     flags =  d3d;
}

var bipedPhy_debug_state = 0;

function bipedPhy_Toggle_Debug()
{
	if (bipedPhy_debug_state)
	{
		bipedPhy_debug_pan.visible = off;
		bipedPhy_debug_txt.visible = off;
		bipedPhy_debug_state = 0;
	} 
	else
	{
		bipedPhy_debug_pan.visible = on;
		bipedPhy_debug_txt.visible = on;
		bipedPhy_debug_state = 1;
	}
}

function bipedPhy_startup()
{
	gl_debug_toggles[gl_debug_max] = handle(bipedPhy_Toggle_Debug);
	gl_debug_max += 1;
}

endif;

// =====================================================================
// MISC FUNCTIONS

// Calculate the damage from a fall and return that value
function	bipedPhy_Fall_Damage()
{
	if (my.GL_no_fall_dam == 1) 
	{ 
		return(0); 
	}
	if (my.GL_Ironman == 1) 
	{ 
		return(0); 
	}
	if (my.GL_biped_state2_value - bipedPhy_safe_fall_time_const < 0)
	{
		return(0);
	}
	// calculate damage depending on fall time
 	return(INT(((my.GL_biped_state2_value - bipedPhy_safe_fall_time_const) * bipedPhy_fall_damage_const)));
}



// =====================================================================
// SETUP FUNCTIONS


// Desc: Setup an entity so it can use bipedPhy_Update
//
// NOTE: make sure the entity has been loaded at least one frame before calling (wait(1)).
function bipedPhy_Setup_Ent()
{
	my.fat = ON; my.narrow = ON;
	c_setminmax(my);
	// make symmetrical in X/Y
	GL_vec1.x = max(-my.min_x,my.max_x);
	GL_vec1.y = max(-my.min_y,my.max_y);
	GL_vec1.z = max(GL_vec1.x,GL_vec1.y);
	my.min_x = -GL_vec1.z; 
	my.max_x = GL_vec1.z;
	my.min_y = -GL_vec1.z; 
	my.max_y = GL_vec1.z;

}



// Desc: handle biped panning with absolute values (not time correct)
//
// Input: ang_dist (vec) set to the panning amount (modified)
//
// Result:
//		Sets: my._gid01_ang_vel_pan equal to the angular velocity
// Assume: my is set to a biped01 entity
function bipedPhy_Pan(ang_dist)
{
	var ang_tmp;
	ang_tmp = c_rotate(my,vector(ang_dist,0,0),
		ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box);

	my.GL_Pan_Vel = ang_tmp / time_step; // calculate ang vel for use later
ifdef	develop;
	if (my.gl_entity_id == gl_player1)
	{
		bipedPhy_debug_data[20] = ang_dist;
		bipedPhy_debug_data[21] = ang_tmp;
	}
endif;
}


//==============================================================================================================================================

// desc: simple scan. Does NOT detect passable blocks (water,forcefields, etc.)
//
// Assume: my is set to a valid biped01 entity (see doc for valid skills)
var bipedPhy_to_vec[3];   // can't use local vecs when calling functions (v6.2)
var bipedPhy_from_vec[3];
function	bipedPhy_Scan()  // c_collide version
{
	if(my == NULL) 
	{ 
		diag("************NULLED MY IN bipedPhy_Scan****"); 
		return(-1); 
	}

 	// Use trace to scan the surface under the entity
	vec_set(bipedPhy_from_vec,my.x);
	vec_set(bipedPhy_to_vec,my.x);
	bipedPhy_to_vec.z -= bipedPhy_scan_dist_const;
	bipedPhy_height = c_trace(bipedPhy_from_vec,bipedPhy_to_vec,IGNORE_ME + ACTIVATE_SONAR + USE_BOX);


	if(0 == trace_hit) // 0-if nothing hit
	{
		bipedPhy_height = bipedPhy_scan_dist_const;	// greater than x units off the ground (in air)
	}


	bipedPhy_temp_ent = you;	// save you for later use

	vec_set(bipedPhy_surface_normal_vec.x,NORMAL.X); // normal is set in trace


	if(passable_ent == null)
	{
		if(ON_PASSABLE || IN_PASSABLE)
		{
			// first trace returned a passable block
			bipedPhy_height = c_trace(bipedPhy_from_vec,bipedPhy_to_vec,
				IGNORE_ME + ACTIVATE_SONAR + USE_BOX + IGNORE_PASSABLE + IGNORE_PASSENTS);
		}
	}

	// if the player is standing in a forcefield, water, or on a platform
	if(bipedPhy_temp_ent != NULL)
	{
		if(GL_LIFT == bipedPhy_temp_ent.GL_Entity_id)
		{
			// add current platform velocity
			vec_set(bipedPhy_surface_vel_vec[0],bipedPhy_temp_ent.GL_X_Vel);
		}
		else
		{
			// no added velocity due to platform
			vec_set(bipedPhy_surface_vel_vec[0],nullvector);
		}
	}
	else
	{
		// no added velocity
		vec_set(bipedPhy_surface_vel_vec[0],nullvector);


	}

	return(0);
}



// Desc: move on the ground, air, or wading
//
// assume: my is set to a biped01
//			  bipedPhy_height is set to the current height above a surface
//			  bipedPhy_grnd_friction is set to the ground friction value
function	bipedPhy_Move_Ground()
{

	var	friction;	// friction constant
	var	delta_fric; // friction over time value
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	abs_force[3];//forces from the environment (gravity, elevators, etc.)
	var	q_jump_or_duck; // 1: jump, -1: duck, 0: neither
	var	temp_ang;
	
	// jump or duck?
	q_jump_or_duck = sign(bipedPhy_force_vec[2]);
	bipedPhy_force_vec[2] = 0;

	// this biped doesn't tilt or roll
	my.tilt = 0;

	temp_ang = bipedPhy_ang_vec.pan;
	// rotate the player (absolute)
	bipedPhy_Pan(temp_ang);

	///////////////////////////////////////////
	// drop to ground and check for collision


	if(bipedPhy_height<2) // on, in, or near the ground?
	{
		// push out of ground
		// note: With the new collision model, this should never happen (but does, rarely)
		// note2: with new collision improvements, pushing out of the ground can cause more problems then it fixes,
		//    for now it is set to 0.
		vec_set(abs_dist,nullvector);// = 0;//-(bipedPhy_height);
		my.GL_Z_Vel = max(0,my.GL_Z_Vel); // stop any down velocity
	}
	else   // gravity is in effect.
	{
		abs_dist.x = 0; 
		abs_dist.y = 0;
 		abs_dist.z = accelerate(my.GL_Z_Vel,-bipedPhy_gravity,0); 						// accel downward
 	}

	// use the move_min_z to limit the slope to which the biped can stand on.
	// NOTE: REMOVED so we can climb steps
//	if(my._move_min_z__003 >= 0) { move_min_z = min(0.99,(1-my._move_min_z__003)); }    // set the min_z glide value


   // Height used to detect on ground or in air
	if(bipedPhy_height > 3)
	{
		bipedPhy_height = bipedPhy_scan_dist_const;	// Still in the air

		friction = bipedPhy_air_friction; // use air friction

   	// airborne - reduce all relative forces
		// to prevent him from jumping or further moving in the air
		bipedPhy_force_vec[0] *= 0.2; // don't set the force completely to zero, otherwise
		bipedPhy_force_vec[1] *= 0.2; // player could be stuck on top of a non-wmb entity
		bipedPhy_force_vec[2] = 0;

	//--	vec_set(abs_force,nullvector);  // gravity is handled above.


 		// only falling if moving downward (but not if ducking)
		if(my.GL_Z_Vel < 0)
		{
			// check to make sure we are not in another state
			if(my.GL_biped_state2 != GL_biped_state2_fall_const)
			{
				my.GL_biped_state2 = GL_biped_state2_fall_const; // set falling flag
				my.GL_biped_state2_value = 0; // reset falling time
			}
			my.GL_biped_state2_value += time_step;   // add falling time
		}


	}
	else    // on the Ground
	{
		bipedPhy_height = 0;
		friction = bipedPhy_grnd_friction; // use ground friction

		//-------------------------------------------------------------------------------------------------------------------------
		// Calculate falling damage
 		if((my.GL_biped_state2 == GL_biped_state2_fall_const) &&  // we are falling
			(my.GL_biped_state2_value > bipedPhy_safe_fall_time_const))	// we've been falling more then the safe amount of time
  		{
			// indicate damage from environment
			my.GL_biped_state2 = GL_biped_state2_damage_const;
			my.GL_biped_state2_value = bipedPhy_Fall_Damage();		// take damage depending on fall_time
		}
		else
		{
			// note: this works ONLY because no other state can exist at this point
			//(e.g. the player is not underwater and not falling)
			my.GL_biped_state2 = GL_biped_state2_none_const; // reset flag (we are on ground)
			my.GL_biped_state2_value = 0;
		}
		//-------------------------------------------------------------------------------------------------------------------------

 		// reset absolute forces
 //--		vec_set(abs_force,nullvector);



		// do we wish to jump?
		if ((q_jump_or_duck>0) 
		&& (my.GL_biped_jump_target <= 0)    // no jumping from ducking, or if we are already jumping
		&& (bipedPhy_surface_normal_vec.z >= BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Z_Move]) 
		&& (my.GL_biped_state1 != GL_biped_state1_wade_const)) // no jumping from slopes or water
		{
			// start a jump
			my.GL_biped_jump_target = BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Jump_Height]; // jump delta
			my.GL_biped_state1 = GL_biped_state1_jump_const; // set state to jump
		}
		else
		{
			if(q_jump_or_duck<0)
			{
				if ((my.GL_biped_state1 != GL_biped_state1_duck_const)
				&& (my.GL_biped_state1 != GL_biped_state1_crawl_const))
				{
  					my.min_z += BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height];
  					my.max_z -= BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height];
  					my.gl_eye_height = my.max_z - 9;
				}
				if (my.GL_biped_state1 == GL_biped_state1_duck_const)
				{
					my.GL_biped_state1 = GL_biped_state1_crawl_const; // set state to duck
				}
				else
				{
					my.GL_biped_state1 = GL_biped_state1_duck_const; // set state to duck
				}
 				vec_scale(bipedPhy_force_vec,0.5);
			}
			else
			{
				if (((my.GL_biped_state1 == GL_biped_state1_duck_const)
				|| (my.GL_biped_state1 == GL_biped_state1_crawl_const))
				&& (q_jump_or_duck==0))
				{
					// check to see if we can stand up.
 					// Use trace to scan the surface under the entity
					vec_set(bipedPhy_from_vec,my.x);
					vec_set(bipedPhy_to_vec,my.x);
					bipedPhy_to_vec.z += BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height] + 1;
					c_trace(bipedPhy_from_vec,bipedPhy_to_vec,IGNORE_ME + IGNORE_PASSENTS + USE_BOX);

					if(0 == trace_hit)  // 0 if nothing hit
					{
  						my.min_z -= BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height];
  						my.max_z += BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height];
  						my.gl_eye_height = my.max_z - 9;
  						my.z += BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_Duck_Height];
						my.GL_biped_state1 = GL_biped_state1_stand_const; //++GT
					}
					else
					{
						// my._biped01_state = biped01_state_duck_const;  state is already duck
 						vec_scale(bipedPhy_force_vec,0.5);
					}

				}
			}
		}



	} // end "on the ground"

	// calculate velocities
	dist.x = accelerate(my.GL_X_Vel,bipedPhy_force_vec[0],friction);
	dist.y = accelerate(my.GL_Y_Vel,bipedPhy_force_vec[1],friction);
	dist.z = 0;                      // dz = 0  (only gravity and jumping)


	if(my.GL_biped_jump_target <= 0)	// not jumping
	{
	  //	my._gid01_vel_z = (TIME * abs_force.z) + (delta_fric * my._gid01_vel_z);    // gravity force (old way)
		abs_dist.z += accelerate(my.GL_Z_Vel,abs_force.z,friction);	// new way
		if ((dist.x)
		&& ((my.GL_biped_state1 != GL_biped_state1_duck_const)
		&& (my.GL_biped_state1 != GL_biped_state1_crawl_const)))
		{
			my.GL_biped_state1 = GL_biped_state1_walk_const;
		}
		else
		{
			if ((my.GL_biped_state1 != GL_biped_state1_duck_const)
			&& (my.GL_biped_state1 != GL_biped_state1_crawl_const))
			{
				my.GL_biped_state1 = GL_biped_state1_stand_const;
			}
		}
	}
	else	// jumping
	{
	 	my.GL_Z_Vel = sqrt(my.GL_biped_jump_target*2*bipedPhy_gravity);    // jumping force
		my.GL_biped_jump_target -= my.GL_Z_Vel * time_step;	// reduce the delta
		abs_dist.z += my.GL_Z_Vel * time_step;
	}


	// calculate absolute distance to move
	// NOTE:
	//  - bipedPhy_surface_vel_vec is any velocity added from standing on a moving platform
	abs_dist.x += ((abs_force.x * time) + bipedPhy_surface_vel_vec[0]) * time_step;   // dx = ax*dt^2
	abs_dist.y += ((abs_force.y * time) + bipedPhy_surface_vel_vec[1]) * time_step;   // dy = ay*dt^2
//	abs_dist.z = (my._gid01_vel_z + bipedPhy_surface_vel_vec[2])* TIME;       // dz = vz*dt
	abs_dist.z += (bipedPhy_surface_vel_vec[2])* time_step;       // dz = vz*dt


	// 'normal' move
  //	disable_z_glide = 2; // no sliding down
	result = c_move(my,dist,abs_dist,
		ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide); // ground (and air) move..

  //	move_min_z = 0; // reset value
  //	disable_z_glide = 0;

	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my.GL_biped_dist = vec_length(dist);
		my.GL_biped_dist = min(my.GL_biped_dist,result);
		if(bipedPhy_force_vec[0] < 0) { my.GL_biped_dist *= -1; }	// walking backwards
		temp1 = abs(my.GL_biped_dist/time_step);
		if(temp1 < (BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_runthreshold]*0.75))	// fall back to walking      0.75
		{
			if (my.gl_biped_state1 == GL_biped_state1_run_const)
			{
				my.gl_biped_state1 = GL_biped_state1_walk_const;
			}
		}
		if(temp1 > (BipedAnim_Cycle_times[((my.GL_Animation - 1) * BipedAnim_max_types) + BipedAnim_runthreshold]*1.15))	// keep walking      1.15
		{
			if (my.gl_biped_state1 == GL_biped_state1_walk_const)
			{
				my.gl_biped_state1 = GL_biped_state1_run_const;
			}
		}
		if ((temp1 < BipedAnim_still_threshold) 
		&& (my.GL_biped_state1 != GL_biped_state1_stand_const))
		{ 
			MY.GL_biped_state1 == GL_biped_state1_stand_const;
		}
		if ((temp1 > BipedAnim_still_threshold) 
		&& (my.GL_biped_state1 == GL_biped_state1_stand_const))
		{ 
			MY.GL_biped_state1 == GL_biped_state1_walk_const;
		}
	}
	else
	{
		// player is not moving, do not animate
		my.GL_biped_dist = 0;
	}
ifdef	develop;
if (my.gl_entity_id == gl_player1)
{
	vec_set(bipedPhy_debug_data[13],dist);
	vec_set(bipedPhy_debug_data[16],abs_dist);
	bipedPhy_debug_data[19] = my.GL_biped_dist;
}
endif;

}


// Desc: move in water (passable) surfaces
function	bipedPhy_Move_Water()
{
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	q_up_or_down; // 1: up, -1: down, 0: neither
	var	temp_height1;
	var	temp_height2;
	
	// up or down?
	q_up_or_down = sign(bipedPhy_force_vec[2]);


	//test for wading. if the water is lessthan scan_dist_const depth then wade
	vec_set(bipedPhy_from_vec,my.x);
	bipedPhy_from_vec.z = my.max_z;
	vec_set(bipedPhy_to_vec,my.x);
	bipedPhy_to_vec.z -= bipedPhy_scan_dist_const;
	temp_height1 = c_trace(bipedPhy_from_vec,bipedPhy_to_vec,IGNORE_ME + USE_BOX);
	vec_set(bipedPhy_from_vec,my.x);
	bipedPhy_from_vec.z = my.max_z;
	vec_set(bipedPhy_to_vec,my.x);
	bipedPhy_to_vec.z -= bipedPhy_scan_dist_const * 2;
	temp_height2 = c_trace(bipedPhy_from_vec,bipedPhy_to_vec,ignore_passable + ignore_passents +IGNORE_ME + USE_BOX);

	if (temp_height2) {
		if ((temp_height2 - temp_height1) < bipedPhy_scan_dist_const)
		{
			my.GL_biped_state1 = GL_biped_state1_wade_const;
			abs_dist.z = accelerate(my.GL_Z_Vel,-bipedPhy_gravity,0);	
		}
		else
		{
			my.GL_biped_state1 = GL_biped_state1_swim_const;
		}
	}
	else
	{
		my.GL_biped_state1 = GL_biped_state1_swim_const;
	}
	if (my.GL_Ironman == on)
	{
		my.GL_biped_state1 = GL_biped_state1_wade_const;
		abs_dist.z = accelerate(my.GL_Z_Vel,-bipedPhy_gravity,0);	
	}

	// rotate the player (absolute)
	bipedPhy_Pan(bipedPhy_ang_vec.pan);

	vec_set(abs_dist.x,nullvector);

	// calculate velocities
	vec_accelerate(dist.x, my.GL_X_Vel, bipedPhy_force_vec,bipedPhy_water_friction);

 	// calculate absolute distance to move
	// NOTE:
	//  - we are only concerned about the entity forces here
	if(bipedPhy_content_ent != NULL)
	{
		vec_add(abs_dist.x,bipedPhy_content_ent.GL_X_Vel);
		vec_scale(abs_dist.x,time_step);
	}


	// check for surface swimming
	if(ON_PASSABLE == ON)
	{
		// add in gravity
		abs_dist.z += accelerate(my.GL_Z_Vel,-bipedPhy_gravity,0);

		// no upward forces
	   abs_dist.z = min(0,abs_dist.z);
		dist.z = min(0,dist.z);
	}
	else
	{
		// Reduce any speed from falling
		abs_dist.z += accelerate(my.GL_Z_Vel,0,0.85); // reduce speed
	}

	result = c_move(my,dist,abs_dist,
		ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide); // ground (and air) move..

// 	move_min_z = 0; // reset value

	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my.GL_biped_dist = vec_length(dist);
		my.GL_biped_dist = min(my.GL_biped_dist,result);
		if(bipedPhy_force_vec[0] < 0) { my.GL_biped_dist *= -1; }	// swimming backwards
	}
	else
	{
		// player is not moving, do not animate
		my.GL_biped_dist = 0;
	}
ifdef	develop;
if (my.gl_entity_id == gl_player1)
{
	vec_set(bipedPhy_debug_data[13],dist);
	vec_set(bipedPhy_debug_data[16],abs_dist);
	bipedPhy_debug_data[19] = my.GL_biped_dist;
}
endif;

}


// =====================================================================
// UPDATE FUNCTION

// Desc: update a biped entity for a frame
//
// Assume: 'my' is set to a valid biped01 entity (see doc for valid skills)
function	bipedPhy_Update(&force_in_vec, &ang_in_vec)
{
	var	biped_content;	// 1-empty, 2-passable (water), 3-solid

	vec_set(bipedPhy_force_vec,force_in_vec);
	vec_set(bipedPhy_ang_vec,ang_in_vec);
	if(my.GL_biped_state1 == GL_biped_state1_override_const)
	{
		// movement is being overriden at this time
		return(GL_biped_state1_override_const);
	}

	// if we haven't set a state yet, set the state to standing (default state)
	if(my.GL_biped_state1 == GL_biped_state1_none_const)
	{
		my.GL_biped_state1 = GL_biped_state1_stand_const;
	}


	// scan floor and check to see if we are in solid, passable, etc.
	// sets: bipedPhy_height: height above nearest surface
	//		   bipedPhy_surface_normal_vec[3]: normal vector of surface
	biped_content = bipedPhy_Scan();


	// select rotation and move function by content
 //---	c_content(my.x,0);
	if((IN_PASSABLE == ON) || (ON_PASSABLE))
	{
		bipedPhy_content_ent = passable_ent;//you;
	}
	else
	{
		bipedPhy_content_ent = NULL;
	}

	if(bipedPhy_content_ent != null)
	{
		if(GL_WATER == bipedPhy_content_ent.GL_Entity_id)
		{
			beep();
			bipedPhy_Move_Water();

		}
		else
		{
   		bipedPhy_Move_Ground();
		}
	}
	else
	{
   	bipedPhy_Move_Ground();
	}
ifdef	develop;
if (my.gl_entity_id == gl_player1)
{
	bipedPhy_debug_data[0] = bipedPhy_height;
	bipedPhy_debug_data[1] = trace_hit;
	bipedPhy_debug_data[2] = my.GL_biped_state1;
	bipedPhy_debug_data[3] = 100 * IN_SOLID + 10 * ON_PASSABLE + IN_PASSABLE;
	vec_set( bipedPhy_debug_data[4],bipedPhy_surface_normal_vec);
	vec_set( bipedPhy_debug_data[7],bipedPhy_force_vec);
	vec_set(bipedPhy_debug_data[10],bipedPhy_ang_vec);
}
endif;
}

endif;// bipedPhy_wdl