ifndef bipedSnd_wdl;
define bipedSnd_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
//
// title: Biped Sounds 
// class: SOUND
// type: SUPPORT
// image: bipedSnd.pcx
// help: Handle the playing of sounds for biped models.
//
// needs: bipedSnd.wdl
// uses:
//
// prefix: bipedSnd_
// idcode: 027
//
// This is mostly the same but added sound based on texture flags 
// Added damage sound
// added dying sound
// 
// ENDHEADER
// ---------------------------------------------------------------------




// ---------------------------------------------------------------------
// section: Walking Sound

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 1
sound		bipedSnd_step1_snd0 = <step.wav>;
// entry: Cycle Distance
// help: Distance between step noises (in quants)
// id: 2
define	bipedSnd_step1_dist0 = 36;
// entry: Volume
// help: Volume of step sound noise
// id: 3
define	bipedSnd_step1_vol0 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 4
sound		bipedSnd_step1_snd1 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 5
define	bipedSnd_step1_vol1 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 6
sound		bipedSnd_step1_snd2 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 7
define	bipedSnd_step1_vol2 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 81
sound		bipedSnd_step1_snd3 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 9
define	bipedSnd_step1_vol3 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 10
sound		bipedSnd_step1_snd4 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 11
define	bipedSnd_step1_vol4 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 12
sound		bipedSnd_step1_snd5 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 13
define	bipedSnd_step1_vol5 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 14
sound		bipedSnd_step1_snd6 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 15
define	bipedSnd_step1_vol6 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 16
sound		bipedSnd_step1_snd7 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 17
define	bipedSnd_step1_vol7 = 75;

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 18
sound		bipedSnd_step1_snd8 = <step.wav>;
// entry: Volume
// help: Volume of step sound noise
// id: 19
define	bipedSnd_step1_vol8 = 75;

// ---------------------------------------------------------------------
// section: Swimming Sound

// entry: Sound File
// help: Water noise (used for swimming/wading)
// id: 20
sound		bipedSnd_swim_snd1 = <swim.wav>;
// entry: Cycle Distance
// help: Distance between swimming noise
// id: 21
define	bipedSnd_swim_dist1 = 50;
// entry: Volume
// help: Volume of swimming noise
// id: 22
define	bipedSnd_swim_vol1 = 50;

// ---------------------------------------------------------------------
// section: Damage Sound

// entry: Sound File
// entry: Damage noise 1
//id: 23
sound		bipedSnd_damage_snd1 = <damage1.wav>;
// entry: Volume
// help: volume of damage sound noise
// id: 24
define	bipedSnd_damage_vol1 = 75;
// entry: Sound File
// entry: dying noise 1
//id: 25
sound		bipedSnd_dying_snd1 = <damage1.wav>;
// entry: Volume
// help: Volume of dying sound noise
// id: 26
define	bipedSnd_dying_vol1 = 75;


// Desc: Play sounds depending on the movement and state of the biped.
//
//   'my' must be set before calling
//		my._biped01_dist must be set to the distance that the actor has covered (in quants)
function bipedSnd_Update()
{
	// if NOT dead
	if((my.GL_biped_state1 != GL_biped_state1_dying_const) 
	|| (my.GL_biped_state1 != GL_biped_state1_dead_const))
	{
		// Get biped state
		// walking, running, or crawl mode
		if ((my.GL_biped_state1 == GL_biped_state1_walk_const)
		|| (my.GL_biped_state1 == GL_biped_state1_run_const)
		|| (my.GL_biped_state1 == GL_biped_state1_crawl_const)
		|| (my.GL_biped_state1 == GL_biped_state1_stand_const))
		{
			// walking/running sound
			my.GL_Sound_Dist += abs(my.GL_biped_dist);
			if (my.GL_Sound_Dist > bipedSnd_step1_dist0)
			{
				// ++ determine which sound to play..++
				vec_set(GL_vec1, my.x);
				vec_set(GL_vec2, my.x);
				GL_vec1.z -= 200; //allow for large entities as player
				GL_vec1.z += 200; //allow for large entities as player
				
				c_trace(GL_vec2, GL_vec1.x, SCAN_TEXTURE|IGNORE_ME);
				
				if (tex_flag1 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd1,bipedSnd_step1_vol1);
				}
				if (tex_flag2 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd2,bipedSnd_step1_vol2);
				}
				if (tex_flag3 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd3,bipedSnd_step1_vol3);
				}
				if (tex_flag4 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd4,bipedSnd_step1_vol4);
				}
				if (tex_flag5 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd5,bipedSnd_step1_vol5);
				}
				if (tex_flag6 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd6,bipedSnd_step1_vol6);
				}
				if (tex_flag7 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd7,bipedSnd_step1_vol7);
				}
				if (tex_flag8 == on)
				{
					ent_playsound(my,bipedSnd_step1_snd8,bipedSnd_step1_vol8);
				}
				if ((tex_flag1|tex_flag2|tex_flag3|tex_flag4|tex_flag5|tex_flag6|tex_flag7|tex_flag8) == off)
				{
					ent_playsound(my,bipedSnd_step1_snd0,bipedSnd_step1_vol0);
				}
				while (my.GL_Sound_Dist > bipedSnd_step1_dist0)
				{
					// reset value
					my.GL_Sound_Dist -= bipedSnd_step1_dist0;
				}
			}
		}
		if((MY.GL_biped_state1 == GL_biped_state1_wade_const) 
		|| (MY.GL_biped_state1 == GL_biped_state1_swim_const))
		{
			// swimming/wading sound
			my.GL_Sound_Dist += abs(my.GL_biped_dist);
			if (my.GL_Sound_Dist > bipedSnd_swim_dist1)
			{
				// play water sound
				ent_playsound(my,bipedSnd_swim_snd1,bipedSnd_swim_vol1);
				while (my.GL_Sound_Dist > bipedSnd_swim_dist1)
				{
					// reset value
					my.GL_Sound_Dist -= bipedSnd_swim_dist1;
				}
			}
		}
		if((MY.GL_biped_state1 == GL_biped_state1_damage_const)) 
		{
			// damage sound
			if (my.GL_Sound_Last != GL_biped_state1_damage_const)
			{
				ent_playsound(my,bipedSnd_damage_snd1,bipedSnd_swim_vol1);
			}
		}
		if (my.GL_Sound_Dist == 0)
		{
			my.GL_Sound_Dist = 1;
		}
		my.GL_Sound_Last = my.GL_biped_state1;
	}// end if NOT dead
	else
	{
		if (my.GL_Sound_Dist)
		{
			//play only once!
			my.GL_Sound_Dist = 0;
			ent_playsound(my,bipedSnd_dying_snd1,bipedSnd_swim_vol1);	
		}
	}
}



endif; // bipedSnd_wdl