ifndef camsel_wdl;
define camsel_wdl;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.00
// date: 060510
//
// title: Camera Select
// class: CAMERA
// type: SUPPORT
// image: camsel.pcx
// help: new object based camera system.
//
// This uses the new camera select system. (object based)
// class camera {
//		virtual register;
//		virtual update;
//		virtual select;
//		virtual reinit;
// }
//
// ENDHEADER
// ---------------------------------------------------------------------


// New code for views
define CamSel_camera_views_max = 5;
define CamSel_camera_max_types = 10;

// select camera functions
var 		 CamSel_CameraTypes[CamSel_camera_max_types];
function* CamSel_CameraSelect;

var		 CamSel_CameraTypes_Id[CamSel_camera_max_types];
var		 CamSel_MaxCameraType = -1;

//bit field fo user selectable cameras
var		CamSel_User_Selectable;

//update view functions if any
var CamSel_active_camera_update[CamSel_camera_views_max];
function* CamSel_Update;

//current cameras
var		CamSel_active_view_cameras[CamSel_camera_views_max];
view*		CamSel_active_view_camera;
var		CamSel_active_camera_ID[CamSel_camera_views_max];

//carera enityies used for camera target or camera posistion
var		CamSel_ents[CamSel_camera_views_max];
entity*	CamSel_ent;
var		CamSel_tilt[CamSel_camera_views_max];
var		CamSel_roll[CamSel_camera_views_max];

function CamSel_Reinit()
{
	var index;
	index = 0;
		
	while (index < CamSel_camera_views_max)
	{
		CamSel_ents[index] = 0;
		index += 1;
	}
}


function CamSel_Next(ViewIndex)
{
	var id;
	var index;
	
	if (CamSel_User_Selectable == 0)
	{
		//no user selectable camera
		return(0);
	}
	
	id = CamSel_active_camera_ID[ViewIndex];
	if (id != 0)
	{
		id = ((id & 512) >> 9) | ((id & 511) << 1); 
	} 
	else 
	{
		id = 1;
	}
	while(id != CamSel_active_camera_ID[ViewIndex])
	{
		if (id & CamSel_User_Selectable)
		{
			//found next view
			index = 0;
			while (CamSel_CameraTypes_Id[index] != id)
			{
				index += 1;
			}
			CamSel_active_camera_ID[ViewIndex] = id;
			CamSel_CameraSelect = ptr_for_handle(CamSel_CameraTypes[index]);
			if (CamSel_CameraSelect != null) {
				CamSel_CameraSelect(ViewIndex);
			}
			return(id);
		}
		id = ((id & 512) >> 9) | ((id & 511) << 1); 
	}
	return(0);
}

function CamSel_Prev(ViewIndex)
{
	var id;
	var index;
	if (CamSel_User_Selectable == 0)
	{
		//no user selectable camera
		return(0);
	}
	
	id = CamSel_active_camera_ID[ViewIndex];
	if (id != 0)
	{
		id = ((id & 1022) >> 1) | ((id & 1) << 9); 
	} 
	else 
	{
		id = 1;
	}
	while(id != CamSel_active_camera_ID[ViewIndex])
	{
		if (id & CamSel_User_Selectable)
		{
			//found prev view
			index = 0;
			while (CamSel_CameraTypes_Id[index] != id)
			{
				index += 1;
			}
			CamSel_active_camera_ID[ViewIndex] = id;
			CamSel_CameraSelect = ptr_for_handle(CamSel_CameraTypes[index]);
			if (CamSel_CameraSelect != null) {
				CamSel_CameraSelect(ViewIndex);
			}
			return(id);
		}
		id = ((id & 1022) >> 1) | ((id & 1) << 9); 
	}
	return(0);
}

function CamSel_Sel(ViewIndex,id)
{
	var index = 0;
	while (CamSel_CameraTypes_Id[index] != id && CamSel_MaxCameraType >= index)
	{
		index += 1;
	}
	if (index <= CamSel_MaxCameraType)
	{
		// id found
		CamSel_active_camera_ID[ViewIndex] = id;
			CamSel_CameraSelect = ptr_for_handle(CamSel_CameraTypes[index]);
			if (CamSel_CameraSelect != null) {
				CamSel_CameraSelect(ViewIndex);
			}
		return(id);
	}
	return(0);
}

starter CamSel_Init()
{
	var index;
	//wait for a selectable camera to be registered.

	wait(1);	
	CamSel_active_view_cameras[0] = handle(camera);
	
	while (CamSel_MaxCameraType == -1)
	{
		wait(1);
	}
	//select the first camera into the first view
	CamSel_CameraSelect = ptr_for_handle(CamSel_CameraTypes[0]);
	if (CamSel_CameraSelect != null) {
		CamSel_CameraSelect(0);
	}

	//need to reinit between level loads
	GL_Recreate[GL_Recreate_Max] = handle(CamSel_Reinit);
	GL_Recreate_Max += 1;
	
	while (1)
	{
		//wait for game to be loaded before updating the view(s)
		// stop updating the views during level change, restart and pause
		if (GL_Game_state == GL_level_loaded)
		{
			proc_late(); // make sure that this loop runs last
			
			//check mail box slots for messages to change the view.
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] == GL_MAIL_CHANGE_VIEW_NEXT)
			{
				CamSel_Next(0);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW2] == GL_MAIL_CHANGE_VIEW_NEXT)
			{
				CamSel_Next(1);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW3] == GL_MAIL_CHANGE_VIEW_NEXT)
			{
				CamSel_Next(2);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW4] == GL_MAIL_CHANGE_VIEW_NEXT)
			{
				CamSel_Next(3);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW5] == GL_MAIL_CHANGE_VIEW_NEXT)
			{
				CamSel_Next(4);
			}
			
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] == GL_MAIL_CHANGE_VIEW_PREV)
			{
				CamSel_Prev(0);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW2] == GL_MAIL_CHANGE_VIEW_PREV)
			{
				CamSel_Prev(1);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW3] == GL_MAIL_CHANGE_VIEW_PREV)
			{
				CamSel_Prev(2);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW4] == GL_MAIL_CHANGE_VIEW_PREV)
			{
				CamSel_Prev(3);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW5] == GL_MAIL_CHANGE_VIEW_PREV)
			{
				CamSel_Prev(4);
			}
			
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] >= GL_MAIL_CHANGE_VIEW_SEL)
			{
				CamSel_Sel(0, GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] - GL_MAIL_CHANGE_VIEW_SEL);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW2] >= GL_MAIL_CHANGE_VIEW_SEL)
			{
				CamSel_Sel(1, GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] - GL_MAIL_CHANGE_VIEW_SEL);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW3] >= GL_MAIL_CHANGE_VIEW_SEL)
			{
				CamSel_Sel(2, GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] - GL_MAIL_CHANGE_VIEW_SEL);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW4] >= GL_MAIL_CHANGE_VIEW_SEL)
			{
				CamSel_Sel(3, GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] - GL_MAIL_CHANGE_VIEW_SEL);
			}
			if (GL_Mail_Slots[GL_MAIL_CHANGE_VIEW5] >= GL_MAIL_CHANGE_VIEW_SEL)
			{
				CamSel_Sel(4, GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] - GL_MAIL_CHANGE_VIEW_SEL);
			}
			
			//mail messages process so clear mailbox slots
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] = 0;
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW2] = 0;
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW3] = 0;
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW4] = 0;
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW5] = 0;

			index = 0;
			while (index < CamSel_camera_views_max)
			{
				if (CamSel_active_camera_update[index])
				{
					CamSel_Update = ptr_for_handle(CamSel_active_camera_update[index]);
					if (CamSel_Update != null)
					{
						CamSel_Update(index);
					}
				}
				index += 1;
			}
		}
		wait(1);
	}
	
}

function CamSel_Make_View(index, layer, Newaspect)
{
	CamSel_active_view_camera = view_create(layer);
	CamSel_active_view_camera.aspect = Newaspect;
	CamSel_active_view_cameras[index] = handle(CamSel_active_view_camera);
}

function CamSel_Setview_Pos(index, Newpos_x, Newpos_y)
{
	CamSel_active_view_camera = ptr_for_handle(CamSel_active_view_cameras[index]);
	CamSel_active_view_camera.pos_x = Newpos_x;
	CamSel_active_view_camera.pos_y = Newpos_y;
}

function CamSel_Setview_Size(index, Newsize_x, Newsize_y)
{
	CamSel_active_view_camera = ptr_for_handle(CamSel_active_view_cameras[index]);
	CamSel_active_view_camera.size_x = Newsize_x;
	CamSel_active_view_camera.size_y = Newsize_y;
}

function CamSel_Set_Cam_Ent(index, ent)
{
	CamSel_Ent = ent;
	CamSel_Ents[index] = handle(CamSel_Ent);
}

function CamSel_Add_View(index, viewptr)
{
	CamSel_active_view_camera = viewptr;
	CamSel_active_view_cameras[index] = handle(CamSel_active_view_camera);
}


endif;