ifndef FIXEDCAM_WDL;
define FIXEDCAM_WDL;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.00
// date: 060510
//
// title: Fixed Camera
// class: CAMERA
// type: SUPPORT
// image: fixedcam.pcx
// help: have cameras that change and always point at player.
//
// This uses the new camera select system. (object based)
// class camera {
//		virtual register;
//		virtual update;
//		virtual select;
//		virtual reinit;
// }
//
// ENDHEADER
// ---------------------------------------------------------------------

function FixedCam_Select(viewIndex);
function FixedCam_Update(viewIndex);
function FixedCam_Reinit();

// enable: Selectable?
// help: Make camera selectable by user? 
// id: 1
define	FixedCam_user_sel;


define 	FixedCam_Max = 100;
define	_fixedcam_min_dist = skill1;
define   _fixedcam_invisible = flag1;
define   _fixedcam_min_alowed_dist = 200;
define 	_fixedcam_hysteresis = 50;
define   FixedCam_ID = 1;

//camera list
var 		FixedCam_CamList[FixedCam_Max];
var 		FixedCam_MaxCam = 0;
var		FixedCam_Curr_Cam[CamSel_camera_views_max];

entity* 	FixedCam_FixedCam;

// action: FixedCam
// title: Fixed position camera entity
// Desc: Action attached to the camera entity
//
// uses _fixedcam_min_dist, _max_force_y__003
//------------------
// section: Options
//
// help: Used to adjust the activation distance.
// cntl: spin 200 x 1
// skill1: MaxDistance 500
//
// help: Set to make camera visible.
// flag1: Visible 0
//
action FixedCam
{ 
	if (FixedCam_MaxCam >= FixedCam_Max)
	{
		beep();
		diag("\nWDL-Error:FixedCam_Action - To Many Cameras Defined");
		return;
	}
	if (my._fixedcam_invisible == 0) {
		my.invisible = on; 
	}
	if (my._fixedcam_min_dist > 0 && my._fixedcam_min_dist < _fixedcam_min_alowed_dist)
	{
		my._fixedcam_min_dist = _fixedcam_min_alowed_dist;
	}
	my.passable = on; 
	FixedCam_CamList[FixedCam_MaxCam] = handle(me);
	FixedCam_MaxCam += 1;	
} 

function FixedCam_update(ViewIndex)
{
	var ClosestCam;
	var ClosestDist;
	var CurDist;
	var Index;
	
	if (FixedCam_MaxCam == 0)
	{
		return;
	}
	if (CamSel_ents[ViewIndex] == 0)
	{
		return;
	}
	//get distance to current camera
	CamSel_ent = Ptr_For_Handle(CamSel_ents[ViewIndex]);
	CamSel_ent.invisible = off;
	FixedCam_FixedCam = Ptr_For_Handle(FixedCam_CamList[FixedCam_Curr_Cam[ViewIndex]]);
	CurDist = vec_dist(FixedCam_FixedCam.x, CamSel_ent.x);
	ClosestDist = 99999;
	
	// if there is a activation distance set and player is within it then don't change
	if (CurDist < FixedCam_FixedCam._fixedCam_min_dist)
	{
		ClosestCam = FixedCam_Curr_Cam[ViewIndex];
	}
	else
	{
		Index=0;
		while (index < FixedCam_MaxCam)
		{
			if (FixedCam_CamList[index] != NULL)
			{
				FixedCam_FixedCam = Ptr_For_Handle(FixedCam_CamList[index]);
				CurDist = vec_dist(FixedCam_FixedCam.x, CamSel_ent.x);
				// check for within maximum dist to camera for activation.
				if (CurDist <= FixedCam_FixedCam._fixedCam_min_dist - _fixedcam_hysteresis)
				{
					if (ClosestDist > CurDist - _fixedcam_hysteresis) 
					{	
						ClosestDist = CurDist;
						ClosestCam = index;
					}
				}
				else
				{
					// only activate this camera if no max dist
					if (ClosestDist > CurDist - _fixedcam_hysteresis
					&& FixedCam_FixedCam._fixedCam_min_dist == 0)
					{	
						ClosestDist = CurDist;
						ClosestCam = index;
					}
				}
			}
			index += 1;
		}
	}
	FixedCam_Curr_Cam[ViewIndex] = ClosestCam;
	FixedCam_FixedCam = Ptr_For_Handle(FixedCam_CamList[ClosestCam]);
	CamSel_active_view_camera = Ptr_For_Handle(CamSel_active_view_cameras[ViewIndex]);
	
	//set view location to our "camera"
	vec_set(CamSel_active_view_camera.x, FixedCam_FixedCam.x);

	//point the camera at the player
	vec_set(GL_vec1, CamSel_ent.x); 
	vec_sub(GL_vec1, FixedCam_FixedCam.x);
	vec_to_angle(CamSel_active_view_camera.pan, GL_vec1);
}


function FixedCam_Select(viewIndex)
{
	CamSel_active_camera_update[viewIndex] = 	handle(FixedCam_update);
	CamSel_active_camera_ID[viewIndex] = FixedCam_ID;
	if (CamSel_ent)
	{
		CamSel_ent.visible = on;
	}	
}

function FixedCam_Reinit()
{
	var index;
	
	index = 0;
	while (index < FixedCam_Max)
	{
		FixedCam_CamList[index] = null;
		index += 1;
	}
	FixedCam_MaxCam = 0;
}

starter FixedCam_Register()
{
	if (CamSel_MaxCameraType >= CamSel_camera_max_types)
	{
		beep();
		diag("\nWDL-Error:FixedCam_register - To Many Cameras Types");
		return;
	}
	else
	{
		CamSel_MaxCameraType += 1;
		CamSel_CameraTypes[CamSel_MaxCameraType] = handle(FixedCam_Select);
		CamSel_CameraTypes_ID[CamSel_MaxCameraType] = FixedCam_ID;
	}
	
	ifdef FixedCam_user_sel;
	CamSel_User_Selectable |= FixedCam_ID;
	endif;
	
	//need to reinit between level loads
	GL_Recreate[GL_Recreate_Max] = handle(FixedCam_Reinit);
	GL_Recreate_Max += 1;
}

endif;