ifndef plBiped_wdl;
define plBiped_wdl;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.00
// date: 060510
//
// title: Biped 
// class: MOVEMENT
// type: SUPPORT
// image: biped.pcx
// help: main biped action, events and loop.
//
// Fixed small problems
// added cheat code support
// added debug system integration
// improved code throught
//
// ENDHEADER
// ---------------------------------------------------------------------


// entry: ID
// help: Identification number for this player model (used in plSelect)
// help: NOTE: Action not defined if two player models share a single value
// id: 1
define	plBiped_id = 1;


// entry: "Head" offset vector
// help: distance between model center and the 'head' of the player
// help: used for camera view.
// id: 2
var	plBiped_head_offset_vec[3] = 0,0,33;

// ---------------------------------------------------------------------
// section: Activation Scan Segment

// entry: Width
// help: Horizontal width of the activate segment
// id: 3
define	plBiped_activate_scan_horz_width = 60;
// entry: Height
// help: Vertical height of the activate segment
// id: 4
define	plBiped_activate_scan_vert_width = 60;
// entry: Range
// help: Range of activate scan
// id: 5
define	plBiped_activate_scan_range = 100;


// ---------------------------------------------------------------------
// section: Health and Armor Limits

// entry: Max Health
// entry_help: Player can not have more health then this.
// id: 7
define	plBiped_max_health = 100;

// entry: Max Armor
// entry_help: Player can not have more armor then this.
// id: 8
define	plBiped_max_armor = 100;



// ---------------------------------------------------------------------
// section: Damage

// enable: Display Damage panel?
// id: 10
define	plBiped_DAMAGEPAN;
ifdef plBiped_DAMAGEPAN;
// entry: Damage Bitmap
// entry_help: Displayed for a brief time when player takes damage
// id: 9
bmap plBiped_danage_map = <bloodhit.tga>;
endif;

// head angle. used for when view angle changes independently of the player angle
var	plBiped_head_ang_vec[3];
var	plBiped_force_Vec[3];
var	plBiped_ang_force_vec[3];

function plBiped_Give_Health();
function plBiped_Give_Armor();
function plBiped_Give_Ironman();

ifdef develop;
function plBiped_Set_Debug_Data(ent);

text plBiped__debug_txt
{
   font = standard_font;
   layer = 2;
   pos_x = 430;
   pos_y = 10;
   strings = 31;
   string =
		"<- plBiped_force_vec.x",
		"<- plBiped_force_vec.y",
		"<- plBiped_force_vec.z",
		"<- plBiped_ang_force_vec.tilt",
		"<- plBiped_ang_force_vec.pan",
		"<- plBiped_ang_force_vec.roll",
		"<- gl_entity_id",
		"<- GL_X_Vel",
		"<- GL_Y_Vel",
		"<- GL_Z_Vel",
		"<- GL_Pan_Vel",
		"<- GL_Tilt_Vel",
		"<- Gl_Roll_Vel",
		"<- GL_biped_state1",
		"<- GL_biped_state2",
		"<- GL_biped_state2_value",
		"<- GL_biped_state3",
		"<- GL_biped_state3_value",
		"<- GL_biped_jump_target",
		"<- GL_Anim_Dist",
		"<- GL_biped_dist",
		"<- GL_Anim_Old",
		"<- GL_Anim_Cur",
		"<- GL_Anim_Cycle",
		"<- GL_Anim_Percent",
		"<--GL_Anim_OnceOnly",
		"<--GL_Anim_state",
		"<--GL_Sound_Dist",
		"<--GL_Sound_Last",
		"<--",
		"<--l";
   flags = d3d;
}

var  plBiped_debug_data[31];
panel plBiped__debug_pan
{
   pos_x = 400;
   pos_y = 1;

   digits = 0,   9, 5, standard_font, 1, plBiped_debug_data[0];
   digits = 0,  18, 5, standard_font, 1, plBiped_debug_data[1];
   digits = 0,  27, 5, standard_font, 1, plBiped_debug_data[2];
   digits = 0,  36, 5, standard_font, 1, plBiped_debug_data[3];
   digits = 0,  45, 5, standard_font, 1, plBiped_debug_data[4];
   digits = 0,  54, 5, standard_font, 1, plBiped_debug_data[5];
   digits = 0,  63, 5, standard_font, 1, plBiped_debug_data[6];
   digits = 0,  72, 5, standard_font, 1, plBiped_debug_data[7];
   digits = 0,  81, 5, standard_font, 1, plBiped_debug_data[8];
   digits = 0,  90, 5, standard_font, 1, plBiped_debug_data[9];
   digits = 0,  99, 5, standard_font, 1, plBiped_debug_data[10];
   digits = 0, 108, 5, standard_font, 1, plBiped_debug_data[11];
   digits = 0, 117, 5, standard_font, 1, plBiped_debug_data[12];
   digits = 0, 126, 5, standard_font, 1, plBiped_debug_data[13];
   digits = 0, 135, 5, standard_font, 1, plBiped_debug_data[14];
   digits = 0, 144, 5, standard_font, 1, plBiped_debug_data[15];
   digits = 0, 153, 5, standard_font, 1, plBiped_debug_data[16];
   digits = 0, 162, 5, standard_font, 1, plBiped_debug_data[17];
   digits = 0, 171, 5, standard_font, 1, plBiped_debug_data[18];
   digits = 0, 180, 5, standard_font, 1, plBiped_debug_data[19];
   digits = 0, 189, 5, standard_font, 1, plBiped_debug_data[20];
   digits = 0, 198, 5, standard_font, 1, plBiped_debug_data[21];
   digits = 0, 207, 5, standard_font, 1, plBiped_debug_data[22];
   digits = 0, 216, 5, standard_font, 1, plBiped_debug_data[23];
   digits = 0, 225, 5, standard_font, 1, plBiped_debug_data[24];
   digits = 0, 234, 5, standard_font, 1, plBiped_debug_data[25];
   digits = 0, 243, 5, standard_font, 1, plBiped_debug_data[26];
   digits = 0, 252, 5, standard_font, 1, plBiped_debug_data[27];
   digits = 0, 261, 5, standard_font, 1, plBiped_debug_data[28];
   digits = 0, 270, 5, standard_font, 1, plBiped_debug_data[29];
   digits = 0, 279, 5, standard_font, 1, plBiped_debug_data[30];

	flags = d3d, refresh;
}

function plBiped_Set_Debug_Data(ent)
{
	temp_ent = ent;
	plBiped_debug_data[0]	= plBiped_force_vec.x;
	plBiped_debug_data[1]	= plBiped_force_vec.y;
	plBiped_debug_data[2]	= plBiped_force_vec.z;
	plBiped_debug_data[3]	= plBiped_ang_force_vec.pan;
	plBiped_debug_data[4]	= plBiped_ang_force_vec.tilt;
	plBiped_debug_data[5]	= plBiped_ang_force_vec.roll;
	plBiped_debug_data[6]	= temp_ent.gl_entity_id;
	plBiped_debug_data[7]	= temp_ent.GL_X_Vel;
	plBiped_debug_data[8]	= temp_ent.GL_Y_Vel;
	plBiped_debug_data[9]	= temp_ent.GL_Z_Vel;
	plBiped_debug_data[10]	= temp_ent.GL_Pan_Vel;
	plBiped_debug_data[11]	= temp_ent.GL_Tilt_Vel;
	plBiped_debug_data[12]	= temp_ent.Gl_Roll_Vel;
	plBiped_debug_data[13]	= temp_ent.GL_biped_state1;
	plBiped_debug_data[14]	= temp_ent.GL_biped_state2;
	plBiped_debug_data[15]	= temp_ent.GL_biped_state2_value;
	plBiped_debug_data[16]	= temp_ent.GL_biped_state3;
	plBiped_debug_data[17]	= temp_ent.GL_biped_state3_value;
	plBiped_debug_data[18]	= temp_ent.GL_biped_jump_target;
	plBiped_debug_data[19]	= temp_ent.GL_Anim_Dist;
	plBiped_debug_data[20]	= temp_ent.GL_biped_dist;
	plBiped_debug_data[21]	= temp_ent.GL_Anim_Old;
	plBiped_debug_data[22]	= temp_ent.GL_Anim_Cur;
	plBiped_debug_data[23]	= temp_ent.GL_Anim_Cycle;
	plBiped_debug_data[24]	= temp_ent.GL_Anim_Percent;
	plBiped_debug_data[25]	= temp_ent.GL_Anim_OnceOnly;
	plBiped_debug_data[26]	= temp_ent.GL_Anim_state;
	plBiped_debug_data[27]	= temp_ent.GL_Sound_Dist;
	plBiped_debug_data[28]	= temp_ent.GL_Sound_Last;
	plBiped_debug_data[29]	= 0;
	plBiped_debug_data[30]	= 0;
}

var plBiped_debug_state = 0;
function plBiped_Toggle_Debug()
{
	if (plBiped_debug_state)
	{
		plBiped__debug_pan.visible = off;
		plBiped__debug_txt.visible = off;
		plBiped_debug_state = 0;
	} 
	else
	{
		plBiped__debug_pan.visible = on;
		plBiped__debug_txt.visible = on;
		plBiped_debug_state = 1;
	}
}

endif;

function plBiped_startup()
{
ifdef develop;
	gl_debug_toggles[gl_debug_max] = handle(plBiped_Toggle_Debug);
	gl_debug_max += 1;
endif;
	GL_FULL_HEALTH = plBiped_Give_Health;
	GL_FULL_ARMOR = plBiped_Give_Armor;
	GL_GOD_MODE = plBiped_Give_Ironman;
}


ifdef	plBiped_DAMAGEPAN;
panel plBiped_damage_pan { bmap = plBiped_danage_map; layer = 98; flags = refresh, d3d, transparent; }

// Desc: Display damage panel for a brief time
function PlBiped_Display_Damage()
{
	if(plBiped_damage_pan.visible == off)
	{
		clib_scale_panel(plBiped_damage_pan,screen_size.x, screen_size.y);
		plBiped_damage_pan.visible = on;
		plBiped_damage_pan.pos_x = 0;
		plBiped_damage_pan.pos_y = 0;
 		Clib_Panel_Fade(plBiped_damage_pan,75,0,0.25);
	}
}
endif;  // plBiped_DAMAGEPAN

// Desc: React to damage taken (sound, blood, etc.)
function PlBiped_Damage_Reaction()
{
ifdef	plBiped_DAMAGEPAN;
	PlBiped_Display_Damage();
endif;

	return;
}



// ---------------------------------------------------------------------
// section: Death

// entry: Death Bitmap
// entry_help: This is the bmp the screen will fade to on death.
// id: 6
bmap plBiped_death_bmp = <black.pcx>;
panel plBiped_death_pan { bmap = plBiped_death_bmp; layer = 99; flags = refresh,d3d; }

// Desc: fade the camera to black (Simulating death)
function	PlBiped_Death()
{
	// only enter once
	// note: this works because the state is set after this call!
	if (my.GL_biped_state2_value != 0) 
	{ 
		return; 
	}
	my.GL_biped_state2_value = -1;

	clib_scale_panel(plBiped_death_pan,screen_size.x, screen_size.y);
	plBiped_death_pan.pos_x = 0;
	plBiped_death_pan.pos_y = 0;
	plBiped_death_pan.visible = on;
 	Clib_Panel_Fade(plBiped_death_pan,5,100,3);
}



// ---------------------------------------------------------------------
// FUNCTIONS

// Desc: Check to make sure limits are not being exceeded.
function plBiped_CheckLimits()
{
	if (GL_player1_ent)
	{
		if(GL_player1_ent.GL_Health > plBiped_max_health) 
		{ 
			GL_player1_ent.GL_Health = plBiped_max_health; 
		}
		if (GL_player1_ent.GL_Armor > plBiped_max_armor) 
		{ 
			GL_player1_ent.GL_Armor = plBiped_max_armor; 
		}
	}
	if (GL_player2_ent)
	{
		if(GL_player2_ent.GL_Health > plBiped_max_health) 
		{ 
			GL_player2_ent.GL_Health = plBiped_max_health; 
		}
		if (GL_player2_ent.GL_Armor > plBiped_max_armor) 
		{ 
			GL_player2_ent.GL_Armor = plBiped_max_armor; 
		}
	}
}

//cheat code
function plBiped_Give_Health()
{
	if (GL_player1_ent)
	{
		GL_player1_ent.GL_Health = plBiped_max_health;
	} 
	if (GL_player2_ent)
	{
		GL_player2_ent.GL_Health = plBiped_max_health;
	} 
}

function plBiped_Give_Armor()
{
	if (GL_player1_ent)
	{
		GL_player1_ent.GL_Armor = plBiped_max_armor;
	} 
	if (GL_player2_ent)
	{
		GL_player2_ent.GL_Armor = plBiped_max_armor;
	} 
}

function plBiped_Give_Ironman()
{
	if (GL_player1_ent)
	{
		GL_player1_ent.gl_ironman = on;
	} 
	if (GL_player2_ent)
	{
		GL_player2_ent.gl_ironman = on;
	} 
}


// Desc: update the head angle depending on vector
function	plBiped_Head_Update(&ang_vec)
{
	vec_add(plBiped_head_ang_vec,ang_vec);
	// Limits
	if (plBiped_head_ang_vec.tilt > 85) 
	{ 
		plBiped_head_ang_vec.tilt = 85; 
	}
	if (plBiped_head_ang_vec.tilt < -85) 
	{ 
		plBiped_head_ang_vec.tilt = -85; 
	}
	if (plBiped_head_ang_vec.roll > 25) 
	{ 
		plBiped_head_ang_vec.roll = 25; 
	}
	if (plBiped_head_ang_vec.roll < -25) 
	{ 
		plBiped_head_ang_vec.roll = -25; 
	}
	if (abs(ang_vec.roll) == 0)
	{
		//auto return to vertical
		plBiped_head_ang_vec.roll *= 0.75;
		if (plBiped_head_ang_vec.roll < 3)
		{
			plBiped_head_ang_vec.roll = 0;
		}
	}
}


// Desc: update mode (call once per frame while active)
function	plBiped_Update_loop()
{
	var index;
	while(1)
	{
		if ((my.GL_biped_state1 == GL_biped_state1_dying_const)
		|| (my.GL_biped_state1 == GL_biped_state1_dying_const))
		{
			if ((my.GL_biped_state2 == GL_biped_state2_dead_const)
			&& (my.GL_biped_state2_value == 0))
			{
				plBiped_Death();
				
			}
		}
		vec_set(plBiped_force_vec,nullvect);
		vec_set(plBiped_ang_force_vec,nullvect);
		index = GL_Event_Get(GL_MSG_XY);
		if (index >= 0)
		{
			plBiped_force_vec.x = GL_Event_Get_Data1(index);
			plBiped_force_vec.y = GL_Event_Get_Data2(index);
			GL_Event_Remove(index);
		}
		index = GL_Event_Get(GL_MSG_PANTILT);
		if (index >= 0)
		{
			plBiped_ang_force_vec.pan = GL_Event_Get_Data1(index);
			plBiped_ang_force_vec.tilt = GL_Event_Get_Data2(index);
			GL_Event_Remove(index);
		}
		index = GL_Event_Get(GL_MSG_ZROLL);
		if (index >= 0)
		{
			plBiped_force_vec.z = GL_Event_Get_Data1(index);
			plBiped_ang_force_vec.roll = GL_Event_Get_Data2(index);
			GL_Event_Remove(index);
		}
		// handle headmovement (use 'tilt' from player input
		plBiped_ang_force_vec.roll *= my.GL_max_ang_force_roll;	
		plBiped_Head_Update(vector(0,plBiped_ang_force_vec.tilt,plBiped_ang_force_vec.roll));
	
		
		// scale rotation and linear input by the force modifiers
		if(0 == plFpsInput_Mouse_Look)  // not using mouse look...
		{
			if(0 == mouse_moving) // ..and mouse not moving
			{
				plBiped_ang_force_vec.pan *= my.GL_max_ang_force_pan; //* time_step;
			}
		}
		
		plBiped_ang_force_vec.tilt = 0;	// do not use tilt from player input to handle biped movement
		plBiped_force_vec.x *= my.GL_max_force_x;
		plBiped_force_vec.y *= my.GL_max_force_y;
		plBiped_force_vec.z *= my.GL_max_force_y;
	
		// Send forces/flags to biped01 update (which will handle physics, animation, and sound)
		BipedPhy_Update(plBiped_force_vec, plBiped_ang_force_vec);
ifdef develop;
	if (my == gl_player1_ent)
	{
		plBiped_Set_Debug_Data(my);
	}
endif;
	
		// Handle player specific results from biped update
		if((my.GL_biped_state2 == GL_biped_state2_damage_const)
		&& (my.GL_Ironman == off))
		{
			// handle any damage from biped update (falling, crushing, etc)
			my.GL_Health -= my.GL_biped_state2_value; // take damage (bipass armor)
			my.GL_biped_state2 = GL_biped_state2_none_const;	// reset flag
		}
	
		// Handle drowning damage
		if((my.GL_biped_state2 == GL_biped_state2_drown_const)
		&& (my.GL_Ironman == off))
		{
			// handle any damage from drowning (time based)
			my.GL_Health -= GL_biped_state2_drown_const * time_step;
		}
		
		if (my.GL_Health <= 0)
		{
			my.GL_biped_state1 = GL_biped_state1_dying_const;
			my.GL_biped_state2 = GL_biped_state2_dying_const;
		}
	
	
		// Check flags
	
		// Activation scan?
		index = GL_Event_Get(GL_MSG_ACTIVATE);
		if (index >= 0)
		{
			GL_Event_Remove(index);
			my.GL_Event_Message = GL_EVENT_SCAN;
		}
		else // or passive scan?
		{
			my.GL_Event_Message = GL_EVENT_AREA;
		}
		temp[0] = plBiped_activate_scan_horz_width;
		temp[1] = plBiped_activate_scan_vert_width;
		temp[2] = plBiped_activate_scan_range;
		//scan_entity(my.x,temp);
		//if(1 == cameraTarget_avatar_view_q )
		//{
			// use camera center/facing
		//	c_scan(cameraTarget_pos_vec.x,cameraTarget_ang_vec.pan,temp,
		//		IGNORE_ME | IGNORE_YOU | SCAN_ENTS | SCAN_LIMIT );
		//}
		//else
		//{
			// use model center/facing
			c_scan(my.x,my.pan,temp,
				IGNORE_ME | IGNORE_YOU | SCAN_ENTS | SCAN_LIMIT );
		//}
		index = 0;
		while (index < CamSel_camera_views_max)
		{
			CamSel_ent = ptr_for_handle(CamSel_ents[index]);
			if (CamSel_ent != null)
			{
				if (CamSel_Ent.Gl_entity_id == me.gl_entity_id)
				{
					CamSel_tilt[index] = plBiped_head_ang_vec.tilt;
					CamSel_roll[index] = plBiped_head_ang_vec.roll;
				}
			}
			index += 1;
		}

		
		BipedAnim_Update();
		bipedSnd_Update();
		wait(1);
	}
}


// Desc: handle player biped events
function plBiped_Event()
{

	if((EVENT_TYPE == event_shoot) && (you.GL_Event_Message == GL_EVENT_SHOOT))
	{
		if (my.GL_Ironman == off)
		{
			if(my.GL_Armor > 0)
			{
				my.GL_Armor -= you.GL_Event_data;
	
				if(my.GL_Armor < 0)
				{
					// flesh damage after armor damage
					my.GL_Health += my.GL_Armor;
					my.GL_Armor = 0;
					my.GL_biped_state1 = GL_biped_state1_damage_const;
					PlBiped_Damage_Reaction();	// display damage panel
				}
			}
			else
			{  // take flesh damage
				my.GL_Health -= you.GL_Event_data;
				my.GL_biped_state1 = GL_biped_state1_damage_const;
				PlBiped_Damage_Reaction();	// display damage panel
			}
		}
	}

	// handle scan events
	if(EVENT_SCAN == EVENT_TYPE)
	{
		if(GL_EVENT_EXPLODE == you.GL_Event_Message)
		{

			// trace back to the blast source
			if(my.GL_Ironman == off) // ++ fix c_trace(your.x,my.x,(ignore_me+ignore_you+ignore_passable+ignore_passents)) == 0)
			{
				if(my.GL_Armor > 0)
				{
					my.GL_Armor -= you.GL_Event_data;

					if(my.GL_Armor < 0)
					{
						// flesh damage after armor damage
						my.GL_Health += my.GL_Armor;
						my.GL_Armor = 0;
						my.GL_biped_state1 = GL_biped_state1_damage_const;
						PlBiped_Damage_Reaction();	// display damage panel
					}

				}
				else
				{  // take flesh damage
					my.GL_Health -= you.GL_Event_data;
					my.GL_biped_state1 = GL_biped_state1_damage_const;
  					PlBiped_Damage_Reaction();	// display damage panel
				}
			}
		}
	}



	// door/lift hit player
	if((EVENT_TYPE == event_push))
	{
		if((you.GL_Event_Message == GL_EVENT_PUSH))
		{
  			// alert "pusher" we got the event
			you.GL_event_handled = 1;	// send message back to door/lift

			if (my.GL_Ironman == off)
			{
				// take damage
				my.GL_Health -= you.GL_Event_data;
				my.GL_biped_state1 = GL_biped_state1_damage_const;
			}
		}
	}

	if (my.GL_Health <= 0)
	{
		my.GL_biped_state1 = GL_biped_state1_dying_const;
		my.GL_biped_state2 = GL_biped_state2_dying_const;
	}
}



// ---------------------------------------------------------------------
// ACTIONS

///////////////////////////////////////////////////////////////////////
// Entity skill & flag definitions




action	plBiped
{
//watched = my;	// uncomment to 'watch' the player
	if(GL_player1_ent != null && GL_player2_ent != null)
	{
		diag("\nWARNING! plBiped::plBiped- three or more plBiped entities in level.");
		wait(1); ent_remove(me);
		return;
	}

	// setup for collision
	wait(1);
	
	// set global player biped entity pointer to this object
	if (GL_player1_ent == null)
	{
		GL_player1_ent = me;
		my.GL_Entity_id = GL_Player1;
		CamSel_ents[0] = handle(me);
	} 
	else
	{
		if (GL_player2_ent == null)
		{
			GL_player2_ent = me;
			my.GL_Entity_id = GL_Player2;
		}
	}
	if (my.gl_animation <=0)
	{
		my.gl_animation = 1;
	}
	if (my.gl_animation > BipedAnim_max_types)
	{
		my.gl_animation = BipedAnim_max_types;
	}
	
	
	BipedPhy_Setup_Ent();
	BipedAnim_Do_Precalc();

	// set id to player

	/////////////////////////////////////////////////////////////
	// set up player specific variables (use defaults as needed)

	// default health
	if(my.GL_Health == 0) 
	{ 
		my.GL_Health = 100; 
	}
	// default forces
	if (my.GL_max_force_x == 0) 
	{ 
		my.GL_max_force_x = 7.5; 
	}
	if (my.GL_max_force_y == 0) 
	{ 
		my.GL_max_force_y = 3.75; 
	}
	if (my.GL_max_ang_force_pan == 0) 
	{ 
		my.GL_max_ang_force_pan = 10.0; 
	}
	if (my.GL_max_ang_force_roll == 0) 
	{ 
		my.GL_max_ang_force_roll = 10.0; 
	}
	
	if (my.GL_trigger_range == 0)
	{
		my.GL_trigger_range = 5;
	}
	if (my.GL_trigger_range < 0) 
	{ 
		my.GL_trigger_range = 0; 
	}

	my.GL_Eye_Height = my.max_z - 9;
	
	// set up event handler
	my.event = plBiped_Event;
	my.enable_shoot = on;  	// gun shoot
	my.enable_scan = on;		// explosions
	my.enable_push = on;		// stop/take damage from certain doors and platforms
	// Call the biped setup function
	plBiped_Update_loop();
}

action plBiped01()
{
	plBiped();
}


endif;