ifndef _plFpsInput_wdl;
define _plFpsInput_wdl;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.00
// date: 060510
//
// title: FPS Input
// class: Input
// type: SUPPORT
// image: fpsinput.pcx
// help: handle keys and joystick.
//
// Added Auto joystick type detection
// added support for save/load of key settings
// added more keys
// fixed joystick deadzone bug
// added debug integration
// added support for messaging system
// ENDHEADER
// ---------------------------------------------------------------------



var   plFpsInput_Joystick_Type = 0;
var	plFpsInput_Joystick_dead_x = 50;
var   plFpsInput_Joystick_dead_y = 50;
var 	plFpsInput_Joystick_digital = 0;

var   plFpsInput_Block_Input  = 0;
var	plFpsInput_RepeatCount = 8;

var	plFpsInput_Mouse_Look = 1;
var	plFpsInput_Mouse_Look_inv = 1;
var	plFpsInput_force_slow_const = 0.5;
var	plFpsInput_mouse_look_factor = 5.5;


var 	plFpsInput_force_vec[3];
var	plFpsInput_ang_force_vec[3];

var 	plFpsInput_cam_force_vec[3];
var	plFpsInput_cam_ang_force_vec[3];

var	plFpsInput_Locks[20];

string plFpsInput01_tmp_str[64];



function plFpsInput_xscale_joystick(raw_value);
function plFpsInput_yscale_joystick(raw_value);
function plFpsInput_Set_Default_Keys();
function plFpsInput_Set_Key_ScanCodes();
function plFpsInput_Save_Config();
function plFpsInput_Load_Config();
function plFpsInput_Update();

ifdef develop;
function plFpsInput_Set_Debug_Data(user);

text plFpsInput__debug_txt
{
   font = standard_font;
   layer = 2;
   pos_x = 500;
   pos_y = 10;
   strings = 31;
   string =
		"<- Joystick Type",
		"<- Joystick X axis",
		"<- Joystick Y axis",
		"<- Joystick Z axis",
		"<- Joystick PAN axis",
		"<- Joystick ROLL axis",
		"<- Joystick TILT axis",
		"<- Joystick button 1",
		"<- Joystick button 2",
		"<- Joystick button 3",
		"<- Joystick button 4",
		"<- Joystick button 5",
		"<- Joystick button 6",
		"<- Joystick button 7",
		"<- Joystick button 8",
		"<- Joystick button 9",
		"<- Joystick button 10",
		"<- Joystick button 11",
		"<- Joystick button 12",
		"<- Joystick button 13",
		"<- Joystick button 14",
		"<- Joystick button 15",
		"<- Joystick button 16",
		"<- Input State",
		"<- User defined",
		"<--force x",
		"<--force y",
		"<--force z",
		"<--force pan",
		"<--force tilt",
		"<--force roll";
   flags = d3d;
}

var  plFpsInput_debug_data[31];
panel plFpsInput__debug_pan
{
   pos_x = 470;
   pos_y = 1;

   digits = 0,   9, 5, standard_font, 1, plFpsInput_debug_data[0];
   digits = 0,  18, 5, standard_font, 1, plFpsInput_debug_data[1];
   digits = 0,  27, 5, standard_font, 1, plFpsInput_debug_data[2];
   digits = 0,  36, 5, standard_font, 1, plFpsInput_debug_data[3];
   digits = 0,  45, 5, standard_font, 1, plFpsInput_debug_data[4];
   digits = 0,  54, 5, standard_font, 1, plFpsInput_debug_data[5];
   digits = 0,  63, 5, standard_font, 1, plFpsInput_debug_data[6];
   digits = 0,  72, 5, standard_font, 1, plFpsInput_debug_data[7];
   digits = 0,  81, 5, standard_font, 1, plFpsInput_debug_data[8];
   digits = 0,  90, 5, standard_font, 1, plFpsInput_debug_data[9];
   digits = 0,  99, 5, standard_font, 1, plFpsInput_debug_data[10];
   digits = 0, 108, 5, standard_font, 1, plFpsInput_debug_data[11];
   digits = 0, 117, 5, standard_font, 1, plFpsInput_debug_data[12];
   digits = 0, 126, 5, standard_font, 1, plFpsInput_debug_data[13];
   digits = 0, 135, 5, standard_font, 1, plFpsInput_debug_data[14];
   digits = 0, 144, 5, standard_font, 1, plFpsInput_debug_data[15];
   digits = 0, 153, 5, standard_font, 1, plFpsInput_debug_data[16];
   digits = 0, 162, 5, standard_font, 1, plFpsInput_debug_data[17];
   digits = 0, 171, 5, standard_font, 1, plFpsInput_debug_data[18];
   digits = 0, 180, 5, standard_font, 1, plFpsInput_debug_data[19];
   digits = 0, 189, 5, standard_font, 1, plFpsInput_debug_data[20];
   digits = 0, 198, 5, standard_font, 1, plFpsInput_debug_data[21];
   digits = 0, 207, 5, standard_font, 1, plFpsInput_debug_data[22];
   digits = 0, 216, 5, standard_font, 1, plFpsInput_debug_data[23];
   digits = 0, 225, 5, standard_font, 1, plFpsInput_debug_data[24];
   digits = 0, 234, 5, standard_font, 1, plFpsInput_debug_data[25];
   digits = 0, 243, 5, standard_font, 1, plFpsInput_debug_data[26];
   digits = 0, 252, 5, standard_font, 1, plFpsInput_debug_data[27];
   digits = 0, 261, 5, standard_font, 1, plFpsInput_debug_data[28];
   digits = 0, 270, 5, standard_font, 1, plFpsInput_debug_data[29];
   digits = 0, 279, 5, standard_font, 1, plFpsInput_debug_data[30];

	flags = d3d, refresh;
}

function plFpsInput_Set_Debug_Data(user)
{
	plFpsInput_debug_data[0]	= plFpsInput_Joystick_Type;
	plFpsInput_debug_data[1]	= joy_raw.x;
	plFpsInput_debug_data[2]	= joy_raw.y;
	plFpsInput_debug_data[3]	= joy_raw.z;
	plFpsInput_debug_data[4]	= joy_rot.pan;
	plFpsInput_debug_data[5]	= joy_rot.roll;
	plFpsInput_debug_data[6]	= joy_rot.tilt;
	plFpsInput_debug_data[7]	= joy_1;
	plFpsInput_debug_data[8]	= joy_2;
	plFpsInput_debug_data[9]	= joy_3;
	plFpsInput_debug_data[10]	= joy_4;
	plFpsInput_debug_data[11]	= joy_5;
	plFpsInput_debug_data[12]	= joy_6;
	plFpsInput_debug_data[13]	= joy_7;
	plFpsInput_debug_data[14]	= joy_8;
	plFpsInput_debug_data[15]	= joy_9;
	plFpsInput_debug_data[16]	= joy_10;
	plFpsInput_debug_data[17]	= joy_11;
	plFpsInput_debug_data[18]	= joy_12;
	plFpsInput_debug_data[19]	= joy_hat==315 || joy_hat==0 || joy_hat==45;
	plFpsInput_debug_data[20]	= joy_hat==45 || joy_hat==90 || joy_hat==135;
	plFpsInput_debug_data[21]	= joy_hat==135 || joy_hat==180 || joy_hat==225;
	plFpsInput_debug_data[22]	= joy_hat==225 || joy_hat==270 || joy_hat==315;
	plFpsInput_debug_data[23]	= plFpsInput_Block_Input;
	plFpsInput_debug_data[24]	= user;
	plFpsInput_debug_data[25]	= plFpsInput_force_vec.x;
	plFpsInput_debug_data[26]	= plFpsInput_force_vec.y;
	plFpsInput_debug_data[27]	= plFpsInput_force_vec.z;
	plFpsInput_debug_data[28]	= plFpsInput_ang_force_vec.pan;
	plFpsInput_debug_data[29]	= plFpsInput_ang_force_vec.tilt;
	plFpsInput_debug_data[30]	= plFpsInput_ang_force_vec.roll;
}

var plFpsInput_debug_state = 0;
function plFpsInput_Toggle_Debug()
{
	if (plFpsInput_debug_state)
	{
		plFpsInput__debug_pan.visible = off;
		plFpsInput__debug_txt.visible = off;
		plFpsInput_debug_state = 0;
	} 
	else
	{
		plFpsInput__debug_pan.visible = on;
		plFpsInput__debug_txt.visible = on;
		plFpsInput_debug_state = 1;
	}
}
endif;

//-- keyboard movement and action keys - constants used for default settings
string plFpsInput_move1_for_key_c		= "W";
string plFpsInput_move2_for_key_c		= "CUR UP";
string plFpsInput_back1_for_key_c		= "S";
string plFpsInput_back2_for_key_c		= "CUR DN";
string plFpsInput_left1_for_key_c 		= "A";
string plFpsInput_left2_for_key_c 		= "CUR LT";
string plFpsInput_right1_for_key_c 		= "D";
string plFpsInput_right2_for_key_c 		= "CUR RT";
string plFpsInput_duck1_for_key_c 		= "C";
string plFpsInput_duck2_for_key_c 		= "END";
string plFpsInput_jump1_for_key_c 		= "SPACE";
string plFpsInput_jump2_for_key_c 		= "HOME";
string plFpsInput_rotr1_for_key_c  		= "";
string plFpsInput_rotr2_for_key_c 		= "INS";
string plFpsInput_rotl1_for_key_c 		= "";
string plFpsInput_rotl2_for_key_c 		= "DEL";
string plFpsInput_leanl1_for_key_c 		= "Q";
string plFpsInput_leanl2_for_key_c 		= "";
string plFpsInput_leanr1_for_key_c 		= "E";
string plFpsInput_leanr2_for_key_c 		= "";
string plFpsInput_lookup1_for_key_c 	= "R";
string plFpsInput_lookup2_for_key_c 	= "PG UP";
string plFpsInput_lookdn1_for_key_c 	= "F";
string plFpsInput_lookdn2_for_key_c 	= "PG DN";
string plFpsInput_activate1_for_key_c 	= "CTRL";
string plFpsInput_activate2_for_key_c 	= "mouse rt";
string plFpsInput_zoom1_for_key_c 		= "B";
string plFpsInput_zoom2_for_key_c 		= "kp plus";
string plFpsInput_slow1_for_key_c 		= "shiftl";
string plFpsInput_slow2_for_key_c 		= "shiftr";

//-- joystick buttons
string plFpsInput_joyjmp_for_key_c 		= "JOY_3";
string plFpsInput_joyduck_for_key_c 	= "JOY_9";
string plFpsInput_joyleanl_for_key_c 	= "JOY_2";
string plFpsInput_joyleanr_for_key_c 	= "JOY_4";
string plFpsInput_joyactivate_for_key_c = "JOY_1";
string plFpsInput_joyzoom_for_key_c 	= "JOY_10";
string plFpsInput_joyslow_for_key_c 	= "";

//-- view movement keys
string plFpsInput_cam_forward_key_c 	= "I";
string plFpsInput_cam_backward_key_c 	= "K";
string plFpsInput_cam_left_key_c 		= "J";
string plFpsInput_cam_right_key_c 		= "L";
string plFpsInput_cam_up_key_c 			= "Y";
string plFpsInput_cam_down_key_c 		= "H";
string plFpsInput_cam_pan_left_key_c 	= "U";
string plFpsInput_cam_pan_right_key_c 	= "O";
string plFpsInput_cam_tilt_for_key_c 	= "N";
string plFpsInput_cam_tilt_back_key_c 	= "M";
string plFpsInput_cam_roll_cw_key_c 	= "";
string plFpsInput_cam_roll_ccw_key_c 	= "";

string plFpsInput_system_view1_key_c	= "f6";
string plFpsInput_system_view2_key_c	= "f7";
string plFpsInput_system_view3_key_c	= "f8";
string plFpsInput_system_view4_key_c	= "f9";
string plFpsInput_system_menu_key_c		= "esc";
string plFpsInput_system_quick_save_key_c = "f10";
string plFpsInput_system_quick_load_key_c = "f11";
string plFpsInput_system_help_key_c		= "f1";
string plFpsInput_system_console_key_c	= "tab";

ifdef develop;
string plFpsInput_system_debug_key_c		= "f12";  
ifelse;
string plFpsInput_system_debug_key_c		= "";  
endif;

//-- forward movement
string plFpsInput_move1_for_key[32];
var	 plFpsInput_move1_for_scancode 		= 0;

string plFpsInput_move2_for_key[32];
var	 plFpsInput_move2_for_scancode 		= 0;

//-- backward movement
string plFpsInput_back1_for_key[32];
var	 plFpsInput_back1_for_scancode 		= 0;

string plFpsInput_back2_for_key[32];
var	 plFpsInput_back2_for_scancode 		= 0;

//-- Left strafe
string plFpsInput_left1_for_key[32];
var	 plFpsInput_left1_for_scancode 		= 0;

string plFpsInput_left2_for_key[32];
var	 plFpsInput_left2_for_scancode 		= 0;

//-- right strafe
string plFpsInput_right1_for_key[32];
var	 plFpsInput_right1_for_scancode 		= 0;

string plFpsInput_right2_for_key[32];
var	 plFpsInput_right2_for_scancode 		= 0;

//-- duck
string plFpsInput_duck1_for_key[32];
var	 plFpsInput_duck1_for_scancode 		= 0;

string plFpsInput_duck2_for_key[32];
var	 plFpsInput_duck2_for_scancode 		= 0;

//-- jump
string plFpsInput_jump1_for_key[32];
var	 plFpsInput_jump1_for_scancode 		= 0;

string plFpsInput_jump2_for_key[32];
var	 plFpsInput_jump2_for_scancode 		= 0;

//-- turn right
string plFpsInput_rotr1_for_key[32];
var	 plFpsInput_rotr1_for_scancode 		= 0;

string plFpsInput_rotr2_for_key[32];
var	 plFpsInput_rotr2_for_scancode 		= 0;

//-- turn left
string plFpsInput_rotl1_for_key[32];
var	 plFpsInput_rotl1_for_scancode 		= 0;

string plFpsInput_rotl2_for_key[32];
var	 plFpsInput_rotl2_for_scancode 		= 0;

//-- lean left (roll)
string plFpsInput_leanl1_for_key[32];
var	 plFpsInput_leanl1_for_scancode 		= 0;

string plFpsInput_leanl2_for_key[32];
var	 plFpsInput_leanl2_for_scancode 		= 0;

//-- lean right (roll)
string plFpsInput_leanr1_for_key[32];
var	 plFpsInput_leanr1_for_scancode 		= 0;

string plFpsInput_leanr2_for_key[32];
var	 plFpsInput_leanr2_for_scancode 		= 0;

//-- Look Up
string plFpsInput_lookup1_for_key[32];
var	 plFpsInput_lookup1_for_scancode 	= 0;

string plFpsInput_lookup2_for_key[32];
var	 plFpsInput_lookup2_for_scancode 	= 0;

//-- look Down
string plFpsInput_lookdn1_for_key[32];
var	 plFpsInput_lookdn1_for_scancode 	= 0;

string plFpsInput_lookdn2_for_key[32];
var	 plFpsInput_lookdn2_for_scancode 	= 0;

//-- activate
string plFpsInput_activate1_for_key[32];
var	 plFpsInput_activate1_for_scancode 	= 0;

string plFpsInput_activate2_for_key[32];
var	 plFpsInput_activate2_for_scancode 	= 0;

//-- zoom
string plFpsInput_zoom1_for_key[32];
var	 plFpsInput_zoom1_for_scancode 	 	= 0;

string plFpsInput_zoom2_for_key[32];
var	 plFpsInput_zoom2_for_scancode 	 	= 0;

//-- slow
string plFpsInput_slow1_for_key[32];
var	 plFpsInput_slow1_for_scancode 	 	= 0;

string plFpsInput_slow2_for_key[32];
var	 plFpsInput_slow2_for_scancode 	 	= 0;

//-- Joystick specific buttons
// forward + backward will always be taken care of by stick
// turn and look up/down may be taken care of by joystick 
// depends on the joystick type. Need options for
// mouse look only or mouse turn/forward back.
// maybe even third state of mouse aim. make joystick rot/F. B.

//-- Jump
string plFpsInput_joyjmp_for_key[32];
var	 plFpsInput_joyjmp_for_scancode 	 	= 0;

//-- Duck
string plFpsInput_joyduck_for_key[32];
var	 plFpsInput_joyduck_for_scancode 	= 0;

//-- lean left
string plFpsInput_joyleanl_for_key[32];
var	 plFpsInput_joyleanl_for_scancode  	= 0;

//-- lean right
string plFpsInput_joyleanr_for_key[32];
var	 plFpsInput_joyleanr_for_scancode  	= 0;

//-- activate
string plFpsInput_joyactivate_for_key[32];
var    plFpsInput_joyactivate_for_scancode = 0;

//-- zoom
string plFpsInput_joyzoom_for_key[32];
var    plFpsInput_joyzoom_for_scancode  	= 0;

//-- slow
string plFpsInput_joyslow_for_key[32];
var    plFpsInput_joyslow_for_scancode  	= 0;

//-- view forward
string plFpsInput_cam_forward_key[32];
var	plFpsInput_cam_forward_scancode 		= 0;

//-- view backward
string plFpsInput_cam_backward_key[32];
var	plFpsInput_cam_backward_scancode 	= 0;

//-- view left
string plFpsInput_cam_left_key[32];
var	plFpsInput_cam_left_scancode 			= 0;

//-- view right
string plFpsInput_cam_right_key[32];
var	plFpsInput_cam_right_scancode 		= 0;

//-- view up
string plFpsInput_cam_up_key[32];
var	plFpsInput_cam_up_scancode 			= 0;

//-- view down
string plFpsInput_cam_down_key[32];
var	plFpsInput_cam_down_scancode 			= 0;

//-- view pan left
string plFpsInput_cam_pan_left_key[32];
var	plFpsInput_cam_pan_left_scancode 	= 0;

//-- view pan right
string plFpsInput_cam_pan_right_key[32];
var	plFpsInput_cam_pan_right_scancode 	= 0;

//-- view tilt down
string plFpsInput_cam_tilt_for_key[32];
var	plFpsInput_cam_tilt_for_scancode 	= 0;

//-- view tilt up
string plFpsInput_cam_tilt_back_key[32];
var	plFpsInput_cam_tilt_back_scancode 	= 0;

//-- view roll clock wise
string plFpsInput_cam_roll_cw_key[32];
var	plFpsInput_cam_roll_cw_scancode 		= 0;

//-- view roll counter clock wise
string plFpsInput_cam_roll_ccw_key[32];
var	plFpsInput_cam_roll_ccw_scancode 	= 0;


string plFpsInput_system_view1_key[32];
var	plFpsInput_system_view1_scancode 	= 0;

string plFpsInput_system_view2_key[32];
var	plFpsInput_system_view2_scancode 	= 0;

string plFpsInput_system_view3_key[32];
var	plFpsInput_system_view3_scancode 	= 0;

string plFpsInput_system_view4_key[32];
var	plFpsInput_system_view4_scancode 	= 0;

string plFpsInput_system_menu_key[32];
var	plFpsInput_system_menu_scancode 		= 0;

string plFpsInput_system_quick_save_key[32];
var	plFpsInput_system_quick_save_scancode = 0;

string plFpsInput_system_quick_load_key[32];
var	plFpsInput_system_quick_load_scancode = 0;

string plFpsInput_system_debug_key[32];
var	plFpsInput_system_debug_scancode 	= 0;

string plFpsInput_system_help_key[32];
var	plFpsInput_system_help_scancode 	= 0;

string plFpsInput_system_console_key[32];
var	plFpsInput_system_console_scancode 	= 0;



function plFpsInput_xscale_joystick(raw_value)
{
	var tmp;
	if ((raw_value >= -plFpsInput_Joystick_dead_x) && (raw_value <= plFpsInput_Joystick_dead_x))
	{
		return(0);
	}
	if (plFpsInput_Joystick_digital != 0)
	{
		return (-sign(raw_value));
	}
	tmp = -(raw_value - sign(raw_value)*plFpsInput_Joystick_dead_x)/(254 - plFpsInput_Joystick_dead_x);
	return(tmp);
}

function plFpsInput_yscale_joystick(raw_value)
{
	var tmp;
	if ((raw_value >= -plFpsInput_Joystick_dead_y) && (raw_value <= plFpsInput_Joystick_dead_y))
	{
		return(0);
	}
	if (plFpsInput_Joystick_digital != 0)
	{
		return(-sign(raw_value));
	}
	tmp = -(raw_value - sign(raw_value)*plFpsInput_Joystick_dead_y)/(254 - plFpsInput_Joystick_dead_y);
	return(tmp);
}

function plFpsInput_Set_Default_Keys()
{
	//-- forward movement
	str_cpy( plFpsInput_move1_for_key, 		plFpsInput_move1_for_key_c);
	str_cpy( plFpsInput_move2_for_key, 		plFpsInput_move2_for_key_c);
	str_cpy( plFpsInput_back1_for_key, 		plFpsInput_back1_for_key_c);
	str_cpy( plFpsInput_back2_for_key, 		plFpsInput_back2_for_key_c);
	str_cpy( plFpsInput_left1_for_key, 		plFpsInput_left1_for_key_c);
	str_cpy( plFpsInput_left2_for_key, 		plFpsInput_left2_for_key_c);
	str_cpy( plFpsInput_right1_for_key, 	plFpsInput_right1_for_key_c);
	str_cpy( plFpsInput_right2_for_key, 	plFpsInput_right2_for_key_c);
	str_cpy( plFpsInput_duck1_for_key,		plFpsInput_duck1_for_key_c);
	str_cpy( plFpsInput_duck2_for_key, 		plFpsInput_duck2_for_key_c);
	str_cpy( plFpsInput_jump1_for_key, 		plFpsInput_jump1_for_key_c);
	str_cpy( plFpsInput_jump2_for_key, 		plFpsInput_jump2_for_key_c);
	str_cpy( plFpsInput_rotr1_for_key , 	plFpsInput_rotr1_for_key_c);
	str_cpy( plFpsInput_rotr2_for_key, 		plFpsInput_rotr2_for_key_c);
	str_cpy( plFpsInput_rotl1_for_key, 		plFpsInput_rotl1_for_key_c);
	str_cpy( plFpsInput_rotl2_for_key, 		plFpsInput_rotl2_for_key_c);
	str_cpy( plFpsInput_leanl1_for_key,		plFpsInput_leanl1_for_key_c);
	str_cpy( plFpsInput_leanl2_for_key, 	plFpsInput_leanl2_for_key_c);
	str_cpy( plFpsInput_leanr1_for_key, 	plFpsInput_leanr1_for_key_c);
	str_cpy( plFpsInput_leanr2_for_key, 	plFpsInput_leanr2_for_key_c);
	str_cpy( plFpsInput_lookup1_for_key, 	plFpsInput_lookup1_for_key_c);
	str_cpy( plFpsInput_lookup2_for_key, 	plFpsInput_lookup2_for_key_c);
	str_cpy( plFpsInput_lookdn1_for_key, 	plFpsInput_lookdn1_for_key_c);
	str_cpy( plFpsInput_lookdn2_for_key, 	plFpsInput_lookdn2_for_key_c);
	str_cpy( plFpsInput_activate1_for_key, plFpsInput_activate1_for_key_c);
	str_cpy( plFpsInput_activate2_for_key, plFpsInput_activate2_for_key_c);
	str_cpy( plFpsInput_zoom1_for_key, 		plFpsInput_zoom1_for_key_c);
	str_cpy( plFpsInput_zoom2_for_key, 		plFpsInput_zoom2_for_key_c);
	str_cpy( plFpsInput_slow1_for_key, 		plFpsInput_slow1_for_key_c);
	str_cpy( plFpsInput_slow2_for_key, 		plFpsInput_slow2_for_key_c);
	str_cpy( plFpsInput_joyjmp_for_key, 	plFpsInput_joyjmp_for_key_c);
	str_cpy( plFpsInput_joyduck_for_key, 	plFpsInput_joyduck_for_key_c);
	str_cpy( plFpsInput_joyleanl_for_key, 	plFpsInput_joyleanl_for_key_c);
	str_cpy( plFpsInput_joyleanr_for_key, 	plFpsInput_joyleanr_for_key_c);
	str_cpy( plFpsInput_joyactivate_for_key,plFpsInput_joyactivate_for_key_c);
	str_cpy( plFpsInput_joyzoom_for_key, 	plFpsInput_joyzoom_for_key_c);
	str_cpy( plFpsInput_joyslow_for_key, 	plFpsInput_joyslow_for_key_c);
	str_cpy( plFpsInput_cam_forward_key,	plFpsInput_cam_forward_key);
	str_cpy( plFpsInput_cam_backward_key,	plFpsInput_cam_backward_key_c);
	str_cpy( plFpsInput_cam_left_key,		plFpsInput_cam_left_key_c);
	str_cpy( plFpsInput_cam_right_key,		plFpsInput_cam_right_key_c);
	str_cpy( plFpsInput_cam_up_key,			plFpsInput_cam_up_key_c);
	str_cpy( plFpsInput_cam_down_key,		plFpsInput_cam_down_key_c);
	str_cpy( plFpsInput_cam_pan_left_key,	plFpsInput_cam_pan_left_key_c);
	str_cpy( plFpsInput_cam_pan_right_key, plFpsInput_cam_pan_right_key_c);
	str_cpy( plFpsInput_cam_tilt_for_key, 	plFpsInput_cam_tilt_for_key_c);
	str_cpy( plFpsInput_cam_tilt_back_key, plFpsInput_cam_tilt_back_key_c);
	str_cpy( plFpsInput_cam_roll_cw_key,	plFpsInput_cam_roll_cw_key_c);
	str_cpy( plFpsInput_cam_roll_ccw_key, 	plFpsInput_cam_roll_ccw_key_c);
	str_cpy( plFpsInput_system_view1_key,	plFpsInput_system_view1_key_c);
	str_cpy( plFpsInput_system_view2_key,	plFpsInput_system_view2_key_c);
	str_cpy( plFpsInput_system_view3_key,	plFpsInput_system_view3_key_c);
	str_cpy( plFpsInput_system_view4_key,	plFpsInput_system_view4_key_c);
	str_cpy( plFpsInput_system_menu_key,	plFpsInput_system_menu_key_c);
	str_cpy( plFpsInput_system_quick_save_key, plFpsInput_system_quick_save_key_c);
	str_cpy( plFpsInput_system_quick_load_key, plFpsInput_system_quick_load_key_c);
	str_cpy( plFpsInput_system_debug_key,	plFpsInput_system_debug_key_c);
	str_cpy( plFpsInput_system_help_key,	plFpsInput_system_help_key_c);
	str_cpy( plFpsInput_system_console_key,	plFpsInput_system_console_key_c);
}

function plFpsInput_Set_Key_ScanCodes()
{
	//-- forward movement
	plFpsInput_move1_for_scancode = MiscInput_Key_For_Str(plFpsInput_move1_for_key);
	plFpsInput_move2_for_scancode = MiscInput_Key_For_Str(plFpsInput_move2_for_key);
	//--
	plFpsInput_back1_for_scancode = MiscInput_Key_For_Str(plFpsInput_back1_for_key);
	plFpsInput_back2_for_scancode = MiscInput_Key_For_Str(plFpsInput_back2_for_key);
	//--
	plFpsInput_left1_for_scancode = MiscInput_Key_For_Str(plFpsInput_left1_for_key);
	plFpsInput_left2_for_scancode = MiscInput_Key_For_Str(plFpsInput_left2_for_key);
	//--
	plFpsInput_right1_for_scancode = MiscInput_Key_For_Str(plFpsInput_right1_for_key);
	plFpsInput_right2_for_scancode = MiscInput_Key_For_Str(plFpsInput_right2_for_key);
	//--
	plFpsInput_duck1_for_scancode = MiscInput_Key_For_Str(plFpsInput_duck1_for_key);
	plFpsInput_duck2_for_scancode = MiscInput_Key_For_Str(plFpsInput_duck2_for_key);
	//--
	plFpsInput_jump1_for_scancode = MiscInput_Key_For_Str(plFpsInput_jump1_for_key);
	plFpsInput_jump2_for_scancode = MiscInput_Key_For_Str(plFpsInput_jump2_for_key);
	//--
	plFpsInput_rotl1_for_scancode = MiscInput_Key_For_Str(plFpsInput_rotr1_for_key);
	plFpsInput_rotr2_for_scancode = MiscInput_Key_For_Str(plFpsInput_rotr2_for_key);
	//--
	plFpsInput_rotl1_for_scancode = MiscInput_Key_For_Str(plFpsInput_rotl1_for_key);
	plFpsInput_rotl2_for_scancode = MiscInput_Key_For_Str(plFpsInput_rotl2_for_key);
	//--
	plFpsInput_leanl1_for_scancode = MiscInput_Key_For_Str(plFpsInput_leanl1_for_key);
	plFpsInput_leanl2_for_scancode = MiscInput_Key_For_Str(plFpsInput_leanl2_for_key);
	//--
	plFpsInput_leanr1_for_scancode = MiscInput_Key_For_Str(plFpsInput_leanr1_for_key);
	plFpsInput_leanr2_for_scancode = MiscInput_Key_For_Str(plFpsInput_leanr2_for_key);
	//--
	plFpsInput_lookup1_for_scancode = MiscInput_Key_For_Str(plFpsInput_lookup1_for_key);
	plFpsInput_lookup2_for_scancode = MiscInput_Key_For_Str(plFpsInput_lookup2_for_key);
	//--
	plFpsInput_lookdn1_for_scancode = MiscInput_Key_For_Str(plFpsInput_lookdn1_for_key);
	plFpsInput_lookdn2_for_scancode = MiscInput_Key_For_Str(plFpsInput_lookdn2_for_key);
	//-- activate
	plFpsInput_activate1_for_scancode = MiscInput_Key_For_Str(plFpsInput_activate1_for_key);
	plFpsInput_activate2_for_scancode = MiscInput_Key_For_Str(plFpsInput_activate2_for_key);
	//-- zoom
	plFpsInput_zoom1_for_scancode = MiscInput_Key_For_Str(plFpsInput_zoom1_for_key);
	plFpsInput_zoom2_for_scancode = MiscInput_Key_For_Str(plFpsInput_zoom2_for_key);
	//-- slow
	plFpsInput_slow1_for_scancode = MiscInput_Key_For_Str(plFpsInput_zoom1_for_key);
	plFpsInput_slow2_for_scancode = MiscInput_Key_For_Str(plFpsInput_zoom2_for_key);
	
	//--
	plFpsInput_joyjmp_for_scancode = MiscInput_Key_For_Str(plFpsInput_joyjmp_for_key);
	plFpsInput_joyduck_for_scancode = MiscInput_Key_For_Str(plFpsInput_joyduck_for_key);
	plFpsInput_joyleanl_for_scancode = MiscInput_Key_For_Str(plFpsInput_joyleanl_for_key);
	plFpsInput_joyleanr_for_scancode = MiscInput_Key_For_Str(plFpsInput_joyleanr_for_key);
	plFpsInput_joyactivate_for_scancode = MiscInput_Key_For_Str(plFpsInput_joyactivate_for_key);
	plFpsInput_joyzoom_for_scancode  = MiscInput_Key_For_Str(plFpsInput_joyzoom_for_key);
	plFpsInput_joyslow_for_scancode  = MiscInput_Key_For_Str(plFpsInput_joyslow_for_key);

 	// view keys
	plFpsInput_cam_forward_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_forward_key);
	plFpsInput_cam_backward_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_backward_key);
	plFpsInput_cam_left_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_left_key);
	plFpsInput_cam_right_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_right_key);
	plFpsInput_cam_up_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_up_key);
	plFpsInput_cam_down_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_down_key);
	plFpsInput_cam_pan_left_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_pan_left_key);
	plFpsInput_cam_pan_right_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_pan_right_key);
	plFpsInput_cam_tilt_for_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_tilt_for_key);
	plFpsInput_cam_tilt_back_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_tilt_back_key);
	plFpsInput_cam_roll_cw_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_roll_cw_key);
	plFpsInput_cam_roll_ccw_scancode  = MiscInput_Key_For_Str(plFpsInput_cam_roll_ccw_key);


	plFpsInput_system_view1_scancode  = MiscInput_Key_For_Str(plFpsInput_system_view1_key);
	plFpsInput_system_view2_scancode  = MiscInput_Key_For_Str(plFpsInput_system_view2_key);
	plFpsInput_system_view3_scancode  = MiscInput_Key_For_Str(plFpsInput_system_view3_key);
	plFpsInput_system_view4_scancode  = MiscInput_Key_For_Str(plFpsInput_system_view4_key);
	plFpsInput_system_menu_scancode  = MiscInput_Key_For_Str(plFpsInput_system_menu_key);
	plFpsInput_system_quick_save_scancode  = MiscInput_Key_For_Str(plFpsInput_system_quick_save_key);
	plFpsInput_system_quick_load_scancode  = MiscInput_Key_For_Str(plFpsInput_system_quick_load_key);
	plFpsInput_system_debug_scancode  = MiscInput_Key_For_Str(plFpsInput_system_debug_key);
	plFpsInput_system_help_scancode  = MiscInput_Key_For_Str(plFpsInput_system_help_key);
	plFpsInput_system_console_scancode  = MiscInput_Key_For_Str(plFpsInput_system_console_key);
}


function plFpsInput_Save_Config()
{
	var OutFile;
	
	OutFile = file_open_write("movement.cfg");
	file_str_write(OutFile, plFpsInput_move1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_move2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_back1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_back2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_left1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_left2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_right1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_right2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_duck1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_duck2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_jump1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_jump2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_rotr1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_rotr2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_rotl1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_rotl2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_leanl1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_leanl2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_leanr1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_leanr2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_lookup1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_lookup2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_lookdn1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_lookdn2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_activate1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_activate2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_zoom1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_zoom2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_slow1_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_slow2_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyjmp_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyduck_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyleanl_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyleanr_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyactivate_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyzoom_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_joyslow_for_key); 
	file_str_write(OutFile, " \n");
	
	file_str_write(OutFile, plFpsInput_cam_forward_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_backward_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_left_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_right_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_up_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_down_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_pan_left_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_pan_right_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_tilt_for_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_tilt_back_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_roll_cw_key); 
	file_str_write(OutFile, " \n");
	

	file_str_write(OutFile, plFpsInput_cam_roll_ccw_key); 
	file_str_write(OutFile, " \n");
	


	file_str_write(OutFile, plFpsInput_system_view1_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_view2_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_view3_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_view4_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_menu_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_quick_save_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_quick_load_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_debug_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_help_key); 
	file_str_write(OutFile, " \n");

	file_str_write(OutFile, plFpsInput_system_console_key); 
	file_str_write(OutFile, " \n");

	file_close(OutFile);
}


function plFpsInput_Load_Config()
{
	var InFile;
	
	InFile = file_open_read("movement.cfg");
	if (InFile == 0)
	{
		return(0);
	}
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_move1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_move2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_back1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_back2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_left1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_left2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_right1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_right2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_duck1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_duck2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_jump1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_jump2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_rotr1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_rotr2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_rotl1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_rotl2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_leanl1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_leanl2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_leanr1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_leanr2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_lookup1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_lookup2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_lookdn1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_lookdn2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_activate1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_activate2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_zoom1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_zoom2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_slow1_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_slow2_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyjmp_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyduck_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyleanl_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyleanr_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyactivate_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyzoom_for_key, plFpsInput01_tmp_str);
	
	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_joyslow_for_key, plFpsInput01_tmp_str);
	

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_forward_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_backward_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_left_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_right_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_up_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_down_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_pan_left_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_pan_right_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_tilt_for_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_tilt_back_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_roll_cw_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_cam_roll_ccw_key, plFpsInput01_tmp_str);


	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_view1_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_view2_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_view3_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_view4_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_menu_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_quick_save_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_quick_load_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_debug_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_help_key, plFpsInput01_tmp_str);

	str_cpy(plFpsInput01_tmp_str,""); 
	if (-1 == file_str_read(InFile, plFpsInput01_tmp_str)) 
	{ 
		file_close(InFile);
		return(0);
	} 
	str_trunc(plFpsInput01_tmp_str,1); 
	str_cpy(plFpsInput_system_console_key, plFpsInput01_tmp_str);

	file_close(InFile);
	return(1)
;}

function plFpsInput_startup()
{
	diag("\nWDL-Loaded:plFpsInput.wdl");


	// Init key commands
	if (plFpsInput_Load_Config() == 0)
	{
		plFpsInput_Set_Default_Keys();
		plFpsInput_Save_Config();
	}
	plFpsInput_Set_Key_ScanCodes();

   // 0 - no joystick
	// 1 - dual analog game pad - type 1
	// 2 - flight stick (x,y, pan)
	// 3 - dual analog game pad - type 2
	// 4 - single analog stick (x,y only)
	// 5 - joy_raw = x,y - joy_rot = pan, tilt;
	// 9 - driving wheel
	if(1 <= num_joysticks)
	{
		if ((joy_rot.y == -255 || joy_rot.y == 255) && (joy_rot.z == -255 || joy_rot.z == 255))
		{
			plFpsInput_Joystick_Type = 1;
			if ((joy_rot.x == -255 || joy_rot.x == 255) && (joy_raw.z == -255 || joy_raw.z == 255))
			{
				plFpsInput_Joystick_Type = 4;
			}
		} else
		{
			if (
					(
						(joy_rot.y == -255 || joy_rot.y == 255) 
					&& (joy_rot.z == -255 || joy_rot.z == 255)
					)
					&& (joy_raw.z == -255 || joy_raw.z == 255)
				)
			{
				plFpsInput_Joystick_Type = 2;
			} else 
			{
				if ((joy_rot.x == -255 || joy_rot.x == 255) && (joy_rot.z == -255 || joy_rot.z == 255))
				{
					plFpsInput_Joystick_Type = 3;
				} else
				{
					if ((joy_rot.z == -255 || joy_rot.z == 255) && (joy_raw.z == -255 || joy_raw.z == 255))
					{
						plFpsInput_Joystick_Type = 5;
					} else
					{
						plFpsInput_Joystick_Type = 0;
					}
				}
			}
		} 
	} 
	else 
	{
		plFpsInput_Joystick_Type = 0;	
	}

ifdef develop;
	gl_debug_toggles[gl_debug_max] = handle(plFpsInput_Toggle_Debug);
	gl_debug_max += 1;
endif;
	
	//start input processing loop
	plFpsInput_Block_Input = OFF;
	while(1)
	{
		plFpsInput_Update();
		wait(1);
	}
}

function plFpsInput_Update()
{
	// zero out forces
	vec_set(plFpsInput_force_vec,nullvector);
	vec_set(plFpsInput_ang_force_vec,nullvector);
	vec_set(plFpsInput_cam_force_vec,nullvector);
	vec_set(plFpsInput_cam_ang_force_vec,nullvector);
	

	if(ON == plFpsInput_Block_Input)
	{
		goto(debug_exit);
	}

	//process system keys regardless of level load state.
	if(plFpsInput_Locks[9]<=0)
	{
		if(Key_Pressed(plFpsInput_system_debug_scancode))
		{ 
			Gl_Event_Add(GL_MSG_DEBUG,0,0);
			plFpsInput_Locks[9] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[9] -= time_step;
	}


	if(plFpsInput_Locks[11]<=0)
	{
		if(Key_Pressed(plFpsInput_system_console_scancode))
		{ 
			Gl_Event_Add(GL_MSG_CONSOLE,0,0);
			plFpsInput_Locks[11] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[11] -= time_step;
	}

	if(GL_level_loaded != GL_Game_state)
	{
		goto(debug_exit);
	}
	
	// check for keyboard/mouse_button/joy_button input
	if(Key_Pressed(plFpsInput_move1_for_scancode)
	|| Key_Pressed(plFpsInput_move2_for_scancode))
	{
		plFpsInput_force_vec.x += 1;	// forward force
	}
	if(Key_Pressed(plFpsInput_back1_for_scancode)
	|| Key_Pressed(plFpsInput_back2_for_scancode))
	{
		plFpsInput_force_vec.x -= 1;	// backwards force
	}
	if(Key_Pressed(plFpsInput_left1_for_scancode)
	|| Key_Pressed(plFpsInput_left2_for_scancode))
	{
		plFpsInput_force_vec.y += 1;	// step left force
	}
	if(Key_Pressed(plFpsInput_right1_for_scancode)
	|| Key_Pressed(plFpsInput_right2_for_scancode)){
		plFpsInput_force_vec.y -= 1;	// step right force
	}
	if(Key_Pressed(plFpsInput_jump1_for_scancode)
	|| Key_Pressed(plFpsInput_jump2_for_scancode)
	|| Key_Pressed(plFpsInput_joyjmp_for_scancode))
	{
		plFpsInput_force_vec.z += 1;	// jump up force
	}
	if(Key_Pressed(plFpsInput_duck1_for_scancode)
	|| Key_Pressed(plFpsInput_duck2_for_scancode)
	|| Key_Pressed(plFpsInput_joyduck_for_scancode))
	{
		plFpsInput_force_vec.z -= 1;	// duck down force
	}
	if(Key_Pressed(plFpsInput_rotr1_for_scancode)
	|| Key_Pressed(plFpsInput_rotr2_for_scancode))
	{
		plFpsInput_ang_force_vec.pan -= 1;    // rotate clockwise force
	}
	if(Key_Pressed(plFpsInput_rotl1_for_scancode)
	|| Key_Pressed(plFpsInput_rotl2_for_scancode))
	{
		plFpsInput_ang_force_vec.pan += 1;    // rotate clockwise force
	}
	if(Key_Pressed(plFpsInput_leanl1_for_scancode)
	|| Key_Pressed(plFpsInput_leanl2_for_scancode))
	{
		plFpsInput_ang_force_vec.roll -= 1;
	}
	if(Key_Pressed(plFpsInput_leanr1_for_scancode)
	|| Key_Pressed(plFpsInput_leanr2_for_scancode))
	{
		plFpsInput_ang_force_vec.roll += 1;
	}
	if(Key_Pressed(plFpsInput_lookup1_for_scancode)
	|| Key_Pressed(plFpsInput_lookup2_for_scancode))
	{
		plFpsInput_ang_force_vec.tilt += 1;    // look up
	}
	if(Key_Pressed(plFpsInput_lookdn1_for_scancode)
	|| Key_Pressed(plFpsInput_lookdn1_for_scancode))
	{
		plFpsInput_ang_force_vec.tilt -= 1;    // look down
	}

	if(Key_Pressed(plFpsInput_cam_forward_scancode))
	{
		plFpsInput_cam_force_vec.x += 1;  
	}
	if(Key_Pressed(plFpsInput_cam_backward_scancode))
	{
		plFpsInput_cam_force_vec.x -= 1;    
	}
	if(Key_Pressed(plFpsInput_cam_left_scancode))
	{
		plFpsInput_cam_force_vec.y += 1;  
	}
	if(Key_Pressed(plFpsInput_cam_right_scancode))
	{
		plFpsInput_cam_force_vec.y -= 1; 
	}
	if(Key_Pressed(plFpsInput_cam_up_scancode))
	{
		plFpsInput_cam_force_vec.z += 1;  
	}
	if(Key_Pressed(plFpsInput_cam_down_scancode))
	{
		plFpsInput_cam_force_vec.z -= 1;
	}
	if(Key_Pressed(plFpsInput_cam_pan_left_scancode))
	{
		plFpsInput_cam_ang_force_vec.pan += 1;
	}
	if(Key_Pressed(plFpsInput_cam_pan_right_scancode))
	{
		plFpsInput_cam_ang_force_vec.pan -= 1; 
	}
	if(Key_Pressed(plFpsInput_cam_tilt_for_scancode))
	{
		plFpsInput_cam_ang_force_vec.tilt += 1; 
	}
	if(Key_Pressed(plFpsInput_cam_tilt_back_scancode))
	{
		plFpsInput_cam_ang_force_vec.tilt -= 1;    
	}
	if(Key_Pressed(plFpsInput_cam_roll_cw_scancode))
	{
		plFpsInput_cam_ang_force_vec.roll += 1;    
	}
	if(Key_Pressed(plFpsInput_cam_roll_ccw_scancode))
	{
		plFpsInput_cam_ang_force_vec.roll -= 1;   
	}

	//--------------------------------------------------------------
	// Joystick/pad
	// NOTE: These override the keyboard movement keys
	if(num_joysticks > 0)
	{
		//use += for these operations so that keyboard can be used with
		//joystick plugged in.
		plFpsInput_force_vec.x += plFpsInput_xscale_joystick(joy_raw.y);
		plFpsInput_force_vec.y += plFpsInput_yscale_joystick(joy_raw.x);

		if (5 == plFpsInput_Joystick_Type)
		{
			plFpsInput_ang_force_vec.pan += plFpsInput_xscale_joystick(joy_rot.tilt);
			plFpsInput_ang_force_vec.tilt += plFpsInput_yscale_joystick(joy_rot.pan);
		}

		if (3 == plFpsInput_Joystick_Type)
		{
			plFpsInput_ang_force_vec.pan += plFpsInput_xscale_joystick(joy_raw.z);
			plFpsInput_ang_force_vec.tilt += plFpsInput_yscale_joystick(joy_rot.tilt);
		}

		if (2 == plFpsInput_Joystick_Type)
		{
			plFpsInput_ang_force_vec.pan += plFpsInput_xscale_joystick(joy_rot.pan);
		}

		if (1 == plFpsInput_Joystick_Type)
		{
			plFpsInput_ang_force_vec.pan += plFpsInput_xscale_joystick(joy_rot.pan);
			plFpsInput_ang_force_vec.tilt += plFpsInput_yscale_joystick(joy_raw.z);
		}	
	}

	if(Key_Pressed(plFpsInput_zoom1_for_scancode)
	|| Key_Pressed(plFpsInput_zoom2_for_scancode)
	|| Key_Pressed(plFpsInput_joyzoom_for_scancode))
	{
		if (plFpsInput_Locks[1] == 0)
		{
			GL_Event_Add(GL_MSG_ZOOM,1,0);	// zoom camera
			plFpsInput_Locks[1] = 1;
		}
	}
	else
	{
		if (plFpsInput_Locks[1] != 0)
		{
			GL_Event_Add(GL_MSG_ZOOM,0,0);	// reset to normal
			plFpsInput_Locks[1] = 0;
		}
	}

	//--------------------------------------------------------------
	// Non-auto keys (keys that only activate once every few ticks)

	if(plFpsInput_Locks[0]<=0)
	{
		if(Key_Pressed(plFpsInput_activate1_for_scancode)
		|| Key_Pressed(plFpsInput_activate2_for_scancode)
		|| Key_Pressed(plFpsInput_joyactivate_for_scancode))
		{
			Gl_Event_Add(GL_MSG_ACTIVATE,0,0);
			plFpsInput_Locks[0] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[0] -= time_step;
	}


	if(plFpsInput_Locks[2]<=0)
	{
		if(Key_Pressed(plFpsInput_system_view1_scancode))
		{
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW1] = 1;	
			plFpsInput_Locks[2] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[2] -= time_step;
	}

	if(plFpsInput_Locks[3]<=0)
	{
		if(Key_Pressed(plFpsInput_system_view2_scancode))
		{
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW2] = 1;	
			plFpsInput_Locks[3] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[3] -= time_step;
	}

	if(plFpsInput_Locks[4]<=0)
	{
		if(Key_Pressed(plFpsInput_system_view3_scancode))
		{
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW3] = 1;	
			plFpsInput_Locks[4] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[4] -= time_step;
	}

	if(plFpsInput_Locks[5]<=0)
	{
		if(Key_Pressed(plFpsInput_system_view4_scancode))
		{
			GL_Mail_Slots[GL_MAIL_CHANGE_VIEW4] = 1;	
			plFpsInput_Locks[5] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[5] -= time_step;
	}

	if(plFpsInput_Locks[7]<=0)
	{
		if(Key_Pressed(plFpsInput_system_quick_save_scancode))
		{
			Gl_Event_Add(GL_MSG_QSAVE,0,0);
			plFpsInput_Locks[7] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[7] -= time_step;
	}

	if(plFpsInput_Locks[8]<=0)
	{
		if(Key_Pressed(plFpsInput_system_quick_load_scancode))
		{
			Gl_Event_Add(GL_MSG_QLOAD,0,0);
			plFpsInput_Locks[8] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[8] -= time_step;
	}

	if(plFpsInput_Locks[10]<=0)
	{
		if(Key_Pressed(plFpsInput_system_help_scancode))
		{ 
			Gl_Event_Add(GL_MSG_HELP,0,0);
			plFpsInput_Locks[10] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[10] -= time_step;
	}

	if(plFpsInput_Locks[6]<=0)
	{
		if(Key_Pressed(plFpsInput_system_menu_scancode))
		{
			Gl_Event_Add(GL_MSG_MENU,0,0);
			plFpsInput_Locks[6] = plFpsInput_RepeatCount;
		}
	}
	else
	{
		plFpsInput_Locks[6] -= time_step;
	}


	// mouse input
	if((1==plFpsInput_Mouse_Look) && (ON == enable_mouse))
	{
		plFpsInput_ang_force_vec.pan -= mouse_force.x*plFpsInput_mouse_look_factor;

		if(1==plFpsInput_Mouse_Look_inv)
		{
			plFpsInput_ang_force_vec.tilt += mouse_force.y*plFpsInput_mouse_look_factor;
		}
		else
		{
			plFpsInput_ang_force_vec.tilt -= mouse_force.y*plFpsInput_mouse_look_factor;
		}

	}

	if(vec_length(plFpsInput_force_vec) > 1)
	{
		vec_normalize(plFpsInput_force_vec,1);
	}


	// force modifiers
	if(Key_Pressed(plFpsInput_slow1_for_scancode)
	|| Key_Pressed(plFpsInput_slow2_for_scancode))
	{
		// slow key pressed, reduce walk speed
		vec_scale(plFpsInput_force_vec,plFpsInput_force_slow_const);
	}
	
	//send any movement needed
   if (plFpsInput_force_vec.x != 0 || plFpsInput_force_vec.y != 0)
   {
   	GL_Event_Add(GL_MSG_XY,plFpsInput_force_vec.x,plFpsInput_force_vec.y);
	}
   if (plFpsInput_ang_force_vec.pan != 0 || plFpsInput_ang_force_vec.tilt != 0)
   {
   	GL_Event_Add(GL_MSG_PANTILT,plFpsInput_ang_force_vec.pan,plFpsInput_ang_force_vec.tilt);
	}
   if (plFpsInput_force_vec.z != 0 || plFpsInput_ang_force_vec.roll != 0)
   {
   	GL_Event_Add(GL_MSG_ZROLL,plFpsInput_force_vec.z,plFpsInput_ang_force_vec.roll);
	}
	
   if (plFpsInput_cam_force_vec.x != 0 || plFpsInput_cam_force_vec.y != 0)
   {
   	GL_Event_Add(GL_MSG_CAM_XY,plFpsInput_cam_force_vec.x,plFpsInput_cam_force_vec.y);
	}
   if (plFpsInput_cam_ang_force_vec.pan != 0 || plFpsInput_cam_ang_force_vec.tilt != 0)
   {
   	GL_Event_Add(GL_MSG_CAM_PANTILT,plFpsInput_cam_ang_force_vec.pan,plFpsInput_cam_ang_force_vec.tilt);
	}
   if (plFpsInput_cam_force_vec.z != 0 || plFpsInput_cam_ang_force_vec.roll != 0)
   {
   	GL_Event_Add(GL_MSG_CAM_ZROLL,plFpsInput_cam_force_vec.z,plFpsInput_cam_ang_force_vec.roll);
	}
debug_exit:
ifdef develop;
	plFpsInput_Set_Debug_Data(plFpsInput_Joystick_Type);
endif;

}

endif;
